/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.kex.datex2.allgemein.KonfigurationsModul;
import de.bsvrz.kex.datex2.allgemein.KonfigurationsModuleFactory;
import de.bsvrz.kex.datex2.allgemein.PersistenzModul;
import de.bsvrz.kex.datex2.allgemein.impl.EinfachesKonfigurationsModulFactory;
import de.bsvrz.kex.datex2.allgemein.impl.ErweiterterLoggerDaV;
import de.bsvrz.kex.datex2.allgemein.impl.PersistenzUeberDaV;
import de.bsvrz.kex.datex2.plugins.definition.KExD2Plugin;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList.Argument;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Hauptklasse der SWE; es werden alle Plugins verwaltet, d.h. initialisiert und periodisch zur 
 * Publikation aufgerufen.
 *
 * @author TraffGo Road GmbH
 * @version $Id$
 */
public final class Verwaltung {
  /** Version der SWE. */
  
  private static final String VERSION = "0.1beta";
  
  /** Standard Timeout für die Ausführung von Plugins (in Sekunden). */
  private static final int STANDARD_PLUGIN_TIMEOUT = 300;
  
  /** Verzögerung für den Start der Verarbeitung dynamischer Daten (in Sekunden). */
  private static final int VERZOEGERUNG_START_DYNAMISCH = 30;

  /** Größe des Thread-Pools für Verarbeitung dynamischer Daten. */  
  private static final int ANZAHL_THREADS = 20;

  /** Logger. */  
  private static Debug logger = Debug.getLogger();
  
  /**
   * Verwaltung ist eine statische Klasse und soll nicht instantiiert werden.
   */
  private Verwaltung() {
  }
	
	/**
	   * Start des Programms, es werden alle Plug-Ins initialisiert und 
	   * der Scheduler gestartet.
	   * 
	   * @param args
	   *          Kommandozeilenargumente:
	   *          -datenverteiler=some.url.with.port:0000 
	   *          -benutzer=username 
	   *          -authentifizierung=/some/path/to/loginfile 
	   *          -debugLevelStdErrText=ERROR 
	   *          -debugLevelFileText=FINEST 
	   *          -debugFilePath=/some/path/to/log/
	   *          -konfigurationsPfad=/pfad/zu/plugin-konfigurationsdateien
	   * 
	   */
	
  public static void main(final String[] args) {  
	  ArgumentList argumentList = new ArgumentList(args);

    // Kommandozeilenoptionen zu Version und Hilfe

	  for (String arg : args) {
	    if (("-version".equals(arg)) || ("--version".equals(arg))) {
	      druckeVersion();
	      System.exit(0);
	    }
	    if (("--help".equals(arg)) || ("-hilfe".equals(arg)) || ("-?".equals(arg))) {
        druckeVersion();
        druckeBenutzung();
        System.exit(0);	      
	    }
	  }

	  // Initialisierung Logging
	  
	Debug.init("KexDatexII", argumentList);

  	logger = Debug.getLogger();
  	logger.info("Logger initialisiert.");
  	logger.info(Debug.debugInfo());

  	// Initialisierung Datenverteiler-Verbindung
  	
    ClientDavInterface davConnection = null;

    try {
      davConnection = erzeugeVerbindung(args);
    } catch (Exception e) {
       logger.error(e.getMessage());
       System.exit(2);
    }
 
    // Sonstige Kommandozeilenoptionen und Fehlerbehandlung
    
    String pfadKonfiguration;
    
    try {
      pfadKonfiguration = argumentList.fetchArgument("-konfigurationsPfad").asString();
    } catch (IllegalArgumentException e) {
      pfadKonfiguration = null;
    }

    if (pfadKonfiguration == null || pfadKonfiguration.isEmpty()) {
      logger.error("Fehler in Kommandozeilenargumente: -konfigurationsPfad=/pfad/zu/konfigurationsdateien fehlt. ");
      davConnection.disconnect(true, "Fehlerhafter Aufruf");
      System.exit(1);
    }
    
    Argument[] unusedArguments = argumentList.fetchUnusedArguments();
    if ((unusedArguments != null) && (unusedArguments.length > 0)) {
      logger.error("Fehler in Kommandozeilenargumente: Unbekannte Option(en): " + Arrays.toString(unusedArguments));
      druckeBenutzung();
      davConnection.disconnect(true, "Fehlerhafter Aufruf");
      System.exit(1);
    }

    // Einlesen der Konfigurationen
    
    KonfigurationsModuleFactory cmf = new EinfachesKonfigurationsModulFactory(pfadKonfiguration);
    List<KonfigurationsModul> configurationModules = cmf.instantiiere();
    
    if (configurationModules.isEmpty()) {
      logger.error("Konnte keine Konfigurationsdateien einlesen.");
      davConnection.disconnect(true, "Keine Plugins konfiguriert.");
      System.exit(1);      
    }
    
    // Instantiierung der Plugins auf Basis der Konfigurationen

    List<KExD2Plugin> plugins = new ArrayList<>();

    for (KonfigurationsModul c : configurationModules) {
      try {
        String pluginClassname = c.getData("Plug-in-Klasse", String.class);
        KExD2Plugin plugin = Class.forName(pluginClassname).asSubclass(KExD2Plugin.class).newInstance();
        plugin.setKonfigurationsModul(c);
        PersistenzModul persistenzModul = new PersistenzUeberDaV(davConnection, c);
        plugin.setPersistenzModul(persistenzModul);
        plugin.setDaVConnection(davConnection);
        ErweiterterLoggerDaV log2 = new ErweiterterLoggerDaV(c, davConnection);
        log2.sendeStatus("Plugin startet");
        plugin.setErweiterterLogger(log2);
     
        plugin.initialisiere();

        if (plugin.isInitialisiert()) {
          plugins.add(plugin);
          logger.fine("Plugin hinzugefügt: " + plugin.getClass().getName());
        } else {
          logger.error("Plugin " + c.getContext()
              + " meldet keine korrekte Initialisierung und wird nicht ausgeführt werden."); 
        }
      } catch (Exception e) {
        logger.error("Plugin " + c.getContext() + " konnte nicht initialisiert werden.", e);
      }
    }
    
    // Erstmalige Ausführung der Plugins (statische Daten publizieren)

    for (final KExD2Plugin plugin : plugins) {
      final String context = plugin.getKonfigurationsModul().getContext();
      final Integer pluginTimeoutKonfig = plugin.getKonfigurationsModul().getInteger("Plug-in-Timeout");
      final Integer pluginTimeout = (pluginTimeoutKonfig != null) ? pluginTimeoutKonfig : STANDARD_PLUGIN_TIMEOUT;

      // Führe jedes Plugin in einem eigenen Thread für begrenzte Zeit aus
      
      ExecutorService executor = Executors.newSingleThreadExecutor();
      try {
        executor.submit(new Callable<Boolean>() {
          @Override
          public Boolean call() throws Exception {
            try {
              plugin.getPluginAusfuehrungsAlgorithmus().starte();
            } catch (Exception ex) {
              logger.error("In Kontext '" + context + "': Unbehandelter Fehler: ", ex);
            }
            return Boolean.TRUE;
          }
        }).get(pluginTimeout,  TimeUnit.SECONDS);
      } catch (Exception tex) {
        logger.info("In Kontext '" + context + "': Ausführung des Plugins wegen Zeitüberschreitung abgebrochen.");
      }
      executor.shutdownNow();    
    }        
    
    // Periodische Ausführung der Plugins (dynamische Daten publizieren)
    
    ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(ANZAHL_THREADS);

    for (final KExD2Plugin plugin : plugins) {
      final String context = plugin.getKonfigurationsModul().getContext();
      final Integer pluginTimeoutKonfig = plugin.getKonfigurationsModul().getInteger("Plug-in-Timeout");
      final Integer pluginTimeout = (pluginTimeoutKonfig != null) ? pluginTimeoutKonfig : STANDARD_PLUGIN_TIMEOUT;

      final Integer pluginOffsetKonfig = plugin.getKonfigurationsModul().getInteger("Plug-in-Start-Offset");
      final Integer pluginOffset = (pluginOffsetKonfig != null) ? pluginOffsetKonfig : VERZOEGERUNG_START_DYNAMISCH;

      Runnable runnable = new Runnable() {
        @Override
        public void run() {
          final ExecutorService executor = Executors.newSingleThreadExecutor();
          try {
            executor.submit(new Callable<Boolean>() {
              @Override
              public Boolean call() throws Exception {
                try {
                  plugin.getPluginAusfuehrungsAlgorithmus().aktualisiere();
                } catch (Exception ex) {
                  logger.error("In Kontext '" + context + "': Unbehandelter Fehler: ", ex);
                }
                return Boolean.TRUE;
              }
            }).get(pluginTimeout,  TimeUnit.SECONDS);
          } catch (Exception tex) {
            logger.info("In Kontext '" + context + "': Ausführung des Plugins wegen Zeitüberschreitung abgebrochen.");
          }
          executor.shutdownNow();    
        }        
      };

      scheduler.scheduleAtFixedRate(runnable, pluginOffset, plugin.getPeriod(), TimeUnit.SECONDS);
    }
    
    if (plugins.isEmpty()) {
      logger.error("Kein Plugin zur Ausführung vorhanden. Programm wird beendet.");
      davConnection.disconnect(false, "");
    }
  }
  
  /**
   * Verbinde mit dem Datenverteiler.
   * 
   * @param args
   *          Kommandozeilenargumente
   * @return Verbindung als ClientDavConnection Objekt.
   * @throws MissingParameterException Es fehlen notwendige Parameter.
   * @throws ConnectionException Allgemeiner Verbindungsfehler.
   * @throws CommunicationError Allgemeiner Übertragungsfehler.
   * @throws InconsistentLoginException Fehler beim Login.
   */
  
  protected static ClientDavInterface erzeugeVerbindung(final String[] args) 
      throws MissingParameterException, CommunicationError, ConnectionException, InconsistentLoginException {
	    ClientDavParameters parameters = new ClientDavParameters(args);
	    ClientDavInterface connection = new ClientDavConnection(parameters);
	    connection.connect();
	    connection.login();
	    logger.info("Verbunden mit Datenverteiler.");
	    return connection;
	  }

  /**
   * Ausgabe der Version der Software.
   */
  
  public static void druckeVersion() {
    System.out.println("KEx-Datex II");
    System.out.println("Version: " + VERSION);
    System.out.println();
  }
  

  /**
   * Ausgabe der möglichen Kommandozeilen-Parameter.
   */
  
  public static void druckeBenutzung() {
    System.out.println("Kommandozeilen-Optionen:");
    System.out.println("          -datenverteiler=some.url.with.port:0000"); 
    System.out.println("          -benutzer=username"); 
    System.out.println("          -authentifizierung=/some/path/to/loginfile");
    System.out.println("          -debugLevelStdErrText=ERROR");
    System.out.println("          -debugLevelFileText=FINEST");
    System.out.println("          -debugFilePath=/some/path/to/log/");
    System.out.println("          -konfigurationsPfad=/pfad/zu/plugin-konfigurationsdateien");
  }
}
