/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.datensammlung.impl;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;

import javax.naming.ConfigurationException;

import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.MutableCollection;
import de.bsvrz.dav.daf.main.config.MutableCollectionChangeListener;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.plugins.datensammlung.DatenSammlungsMethode;
import de.bsvrz.kex.datex2.plugins.definition.PluginExport;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Implementiert die folgende Datensammlungsstrategie: es wird immer das jeweils letzte Element
 * verwendet, solange die Daten nicht zu alt sind.
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */

public class ImmerLetztesElement implements DatenSammlungsMethode {
  /** Schlüssel für maximales Datenalter. */
  private static final String MAXIMALES_ALTER_KEY = "Maximales-Alter";

  /** Sammlung lokaler Caches: pro Objekteliste/Attributgruppe/Aspekt wird ein Cache angelegt. */
  private Map<ObjekteAtgAsp, Map<String, ResultData> > cache = new ConcurrentHashMap<>();
  
  /** Referenz auf Plugin-Definitions-Klasse. */
  private final PluginExport plugin;

  /** Maximales Alter der Daten (in Sekunden). */
  private int maximalesAlter;

  /** Logging. */
  private static final Debug LOGGER = Debug.getLogger();
 
  /**
   * Innere Klasse, implementiert den Empfang von Datenverteiler-Daten.
   * 
   */
  public class UpdateHandler implements ClientReceiverInterface, MutableCollectionChangeListener {
    /** Lokaler Cache. */
    private final Map<String, ResultData> lokalerCache;

    /** DataDescription der Subskription. */
    private final DataDescription dataDescription;
    /**
     * Konstruktor.
     * 
     * @param lokalerCache Lokaler Cache.
     * @param dataDescription DataDescription der Subskription.
     */
    UpdateHandler(final Map<String, ResultData> lokalerCache, final DataDescription dataDescription) {
      this.lokalerCache = lokalerCache;    
      this.dataDescription = dataDescription;
    }
    
    @Override
    public void update(final ResultData[] resultData) {
      for (ResultData x : resultData) {
        if ((x != null) && (x.getObject() != null)) {
          lokalerCache.put(x.getObject().getPid(),  x);
        }
      }
    }

    @Override
    public void collectionChanged(final MutableCollection arg0, final short arg1, 
        final List<SystemObject> added, final List<SystemObject> removed) {
      LOGGER.fine("Änderung der dynamischen Menge: +" + added.size() + " -" + removed.size());
      plugin.getDaVConnection().unsubscribeReceiver(
          this,
          removed,
          dataDescription);
      
      plugin.getDaVConnection().subscribeReceiver(
          this,
          added,
          dataDescription,
          ReceiveOptions.normal(),
          ReceiverRole.receiver(),
          1
          );
    }

  }
  
  /**
   * Konstruktor.
   * 
   * @param plugin Referenz auf Plugin-Definitions-Instanz.
   */
  
  public ImmerLetztesElement(final PluginExport plugin) {
    this.plugin = plugin;
  }
  
  /**
   * Es existiert keine explizite Steuerung über die Sendebereitschaft.
   * Die Daten sind (solange welche vorhanden sind) immer sendebereit.
   * 
   * @return true (außer es sind keine Daten vorhanden)
   */
  
  @Override
  public boolean isSendeBereit() {
    return !cache.isEmpty();
  }
  
  /**
   * Kopiere die Daten aus den lokalen Caches in eine einzige Struktur.
   * 
   * @return Mapping Objektliste/Attributgruppe/Aspekt-Kombination auf Liste von Daten.
   */

  @Override
  public Map<ObjekteAtgAsp, Collection<ResultData> > ermittleDaten() {
    LOGGER.fine("Daten werden aus dem Cache geholt");
    bereinigeCache();
    Map<ObjekteAtgAsp, Collection<ResultData> > result = new HashMap<>();
    for (ObjekteAtgAsp x : cache.keySet()) {
      result.put(x, cache.get(x).values());
    }
    return result;
  }
  
  /**
   * Bereinige die gesammelten Daten, entferne alle zu alten Einträge.
   */

  public void bereinigeCache() {
    Calendar maximalesAlterAbsolut = Calendar.getInstance();
    maximalesAlterAbsolut.add(Calendar.SECOND, -maximalesAlter);

    Calendar datenAlterAbsolut = Calendar.getInstance();

    for (ObjekteAtgAsp objekteAtgAsp : cache.keySet()) {
      Iterator<Entry<String, ResultData>> it = cache.get(objekteAtgAsp).entrySet().iterator(); 
      while (it.hasNext()) {
        Entry<String, ResultData> data = it.next();

        datenAlterAbsolut.setTimeInMillis(data.getValue().getDataTime());
        if (datenAlterAbsolut.before(maximalesAlterAbsolut)) {
          it.remove();
        }
      }
    }
  }
  
  /**
   * Kein Handlungsbedarf nach der Publikation, Daten können auch mehrfach verwendet werden.
   */
  @Override
  public void wurdePubliziert() {
  }

  @Override
  public boolean initialisiere(final Collection<ObjekteAtgAsp> objekteAtgAspListe) throws ConfigurationException {
    try {
      maximalesAlter = plugin.getKonfigurationsModul().getInteger(MAXIMALES_ALTER_KEY);
    } catch (Exception ex) {
      throw new ConfigurationException("In Kontext '" + plugin.getKonfigurationsModul() 
          + "': Fehler beim Lesen des Schlüssels '" + MAXIMALES_ALTER_KEY + "': " + ex.getMessage());
    }
    
    for (ObjekteAtgAsp o : objekteAtgAspListe) {
      cache.put(o, new ConcurrentHashMap<String, ResultData>());
 
      UpdateHandler updateHandler = new UpdateHandler(cache.get(o), o.getDataDescription());   

      try {
        plugin.getDaVConnection().subscribeReceiver(
            updateHandler,
            o.getSystemObjekte(),
            o.getDataDescription(),
            ReceiveOptions.normal(),
            ReceiverRole.receiver(),
            1
            );

        if (o.getMutableCollection() != null) {
          o.getMutableCollection().addChangeListener((short) 0, updateHandler);
        }
      } catch (Exception ex) {
        LOGGER.error("Initialisierung der Datensammlung fehlgeschlagen: " + ex.getClass().getName());
        ex.printStackTrace();
        return false;
      }
    }
    return true;
  }      
}
