/*
 * Plug-in KEx DATEX II - Baustellen
 * Copyright (C) 2018 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.elemente.Datex2AlDoderAkDEnum;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.elemente.Datex2ArtDerMassnahmeEnum;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.elemente.Datex2MobilityEnum;
import eu.datex2.schema.dynamisch.baustellen.Mobility;
import eu.datex2.schema.dynamisch.baustellen.Roadworks;
import eu.datex2.schema.dynamisch.baustellen.RoadworksExtended;
import eu.datex2.schema.dynamisch.baustellen.RoadworksExtensionType;
import eu.datex2.schema.dynamisch.baustellen.SituationRecord;
import eu.datex2.schema.dynamisch.baustellen.SubjectTypeOfWorksEnum;
import eu.datex2.schema.dynamisch.baustellen.Subjects;

/**
 * Wrapper f&uuml;r {@link Roadworks}.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: $
 */
public abstract class Datex2Roadworks extends Datex2OperatorAction {

	/**
	 * Konstruktor.
	 * 
	 * @param situationRecord
	 *            {@link SituationRecord}
	 * @param attprefix
	 *            Attribut-Prefix
	 * @param resultData
	 *            {@link ResultData}
	 * @param data
	 *            {@link Data}
	 */
	public Datex2Roadworks(final SituationRecord situationRecord, final String attprefix, final ResultData resultData,
			final Data data) {
		super(situationRecord, attprefix, resultData, data);
	}

	@Override
	public SituationRecord getSituationRecord() throws Datex2BaustelleException {
		final SituationRecord situationRecord = super.getSituationRecord();
		final Roadworks roadworks = (Roadworks) situationRecord;

		// roadworksDuration (optional)
		Data item = getData().getItem(getDavAttributPrefix() + "AlDoderAkD");
		final Datex2AlDoderAkDEnum duration = Datex2AlDoderAkDEnum.getInstance(item.asUnscaledValue().intValue());
		if (duration.getDatexWert() != null) {
			roadworks.setRoadworksDuration(duration.getDatexWert());
		}

		// mobility (optional)
		item = getData().getItem(getDavAttributPrefix() + "MobilOderStationär");
		final Datex2MobilityEnum mobil = Datex2MobilityEnum.getInstance(item.asUnscaledValue().intValue());
		if (mobil.getDatexWert() != null) {
			final Mobility mo = getObjectFactory().createMobility();
			mo.setMobilityType(mobil.getDatexWert());
			roadworks.setMobility(mo);
		}

		// urgentRoadworks
		// siehe MDM_Datenmodell_Baustellen_04-00-00.pdf: Abbildung 20: true bei
		// 'G2'
		item = getData().getItem(getDavAttributPrefix() + "ArtDerMassnahme");
		final Datex2ArtDerMassnahmeEnum datex2ArtDerMassnahmeEnum = getDatex2ArtDerMassnahmeEnum();
		if (datex2ArtDerMassnahmeEnum == Datex2ArtDerMassnahmeEnum.AkuteVerkehrsgefaehrdung) {
			roadworks.setUrgentRoadworks(true);
		}

		// Subjects
		final SubjectTypeOfWorksEnum subjectTypeOfWorksEnum = datex2ArtDerMassnahmeEnum.getSubjectTypeOfWorksEnum();
		if (subjectTypeOfWorksEnum != null) {
			final Subjects subjects = getObjectFactory().createSubjects();
			subjects.setSubjectTypeOfWorks(subjectTypeOfWorksEnum);
			roadworks.setSubjects(subjects);
		}

		// RoadworksExtended (nur bei Bauabschnitten)
		final String rSARegelplan;
		if (isBauabschnitt()) {
			rSARegelplan = getData().getItem(getDavAttributPrefix() + "FahrstreifenGesperrt").getItem("RSARegelplan")
					.asTextValue().getText();
		} else {
			rSARegelplan = "";
		}

		String arbeitsstellenID = getArbeitsstellenID();

		// Entsprechend [MDMProfilBaustellen] soll die Arbeitsstellen-ID
		// „zukünftig“ verwendet werden. Zur Vermeidung von Problemen in der
		// Übergangzeit kann der Export des Elementes unterdrückt werden (siehe
		// 3.2). Wenn der Export freigegeben ist, führt das Modul eine
		// syntaktische Prüfung der Arbeitsstellen-ID durch (siehe
		// [MDMProfilBaustellen], Abbildung 23: Aufbau der Arbeitsstellen-ID am
		// Beispiel). Wenn diese Prüfung fehlschlägt, wird ein Logeintrag
		// (WARNUNG) erzeugt und die entsprechende Baustelle nicht exportiert.
		if (!arbeitsstellenID.isEmpty()) {
			if (ExtraKonfiguration.isArbeitsstellenIDExportieren()) {
				if (!checkArbeitsstellenID(arbeitsstellenID)) {
					throw new Datex2BaustelleException(
							"Die Arbeitstellen-ID '" + arbeitsstellenID + "' ist syntaktisch ungültig");
				}
			} else {
				// nicht exportieren
				arbeitsstellenID = "";
			}
		}

		if (!rSARegelplan.isEmpty() || !arbeitsstellenID.isEmpty()) {
			final RoadworksExtended roadworksExtended = getObjectFactory().createRoadworksExtended();

			if (!arbeitsstellenID.isEmpty()) {
				roadworksExtended.setRoadworksIdentifier(arbeitsstellenID);
			}

			if (!rSARegelplan.isEmpty()) {
				roadworksExtended.setRoadworksLayout(rSARegelplan);
			}
			final RoadworksExtensionType roadworksExtensionType = getObjectFactory().createRoadworksExtensionType();
			roadworksExtensionType.setRoadworksExtended(roadworksExtended);

			roadworks.setRoadworksExtension(roadworksExtensionType);
		}

		return situationRecord;
	}

	private boolean checkArbeitsstellenID(final String arbeitsstellenID) {
		// Aufbau der Arbeitsstellen-ID am Beispiel:
		// 05xxxxA0040_2016003001002

		final String pattern = "\\d\\d\\w\\w\\w\\w\\w\\d\\d\\d\\d\\w\\d\\d\\d\\d\\d\\d\\d\\d\\d\\d\\d\\d\\d";

		return arbeitsstellenID.matches(pattern);
	}

}
