/*
 * Plug-in KEx DATEX II - Baustellen
 * Copyright (C) 2018 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl;

import java.math.BigInteger;
import java.util.Date;
import java.util.List;
import java.util.UUID;

import javax.xml.datatype.XMLGregorianCalendar;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.NumberValue;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.AttributeType;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.elemente.Datex2ArtDerMassnahmeEnum;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.elemente.Datex2FahrstreifenEnum;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.elemente.Datex2MultilingualString;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.elemente.Datex2SeitenstreifenEnum;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.elemente.Datex2SeverityEnum;
import eu.datex2.schema.dynamisch.baustellen.Comment;
import eu.datex2.schema.dynamisch.baustellen.CommentExtended;
import eu.datex2.schema.dynamisch.baustellen.CommentExtensionType;
import eu.datex2.schema.dynamisch.baustellen.CommentType2Enum;
import eu.datex2.schema.dynamisch.baustellen.Impact;
import eu.datex2.schema.dynamisch.baustellen.ImpactExtended;
import eu.datex2.schema.dynamisch.baustellen.ImpactExtensionType;
import eu.datex2.schema.dynamisch.baustellen.LaneRestriction;
import eu.datex2.schema.dynamisch.baustellen.ObjectFactory;
import eu.datex2.schema.dynamisch.baustellen.OverallPeriod;
import eu.datex2.schema.dynamisch.baustellen.ProbabilityOfOccurrenceEnum;
import eu.datex2.schema.dynamisch.baustellen.SituationRecord;
import eu.datex2.schema.dynamisch.baustellen.UrlLink;
import eu.datex2.schema.dynamisch.baustellen.Validity;
import eu.datex2.schema.dynamisch.baustellen.ValidityStatusEnum;

/**
 * Wrapper f&uuml;r {@link SituationRecord}.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: $
 */

public abstract class Datex2SituationRecord implements Datex2BaustelleExportIF {

	private SituationRecord situationRecord;
	private ResultData resultData;
	private Data data;
	private String attPrefix;

	private Datex2Ortsreferenzen ortsReferenzen;

	private ObjectFactory datex2ObjectFactory = new ObjectFactory();

	/**
	 * Erzeugt ein neues Objekt.
	 * 
	 * @param situationRecord
	 *            {@link SituationRecord}
	 * @param attprefix
	 *            DAV-Attribut-Prefix ('Gm' oder 'Ba')
	 * @param resultData
	 *            zugeh&ouml;riges {@link ResultData}
	 * @param data
	 *            zugeh&ouml;riges {@link Data}
	 */
	public Datex2SituationRecord(final SituationRecord situationRecord, final String attprefix,
			final ResultData resultData, final Data data) {
		this.resultData = resultData;
		this.data = data;
		this.situationRecord = situationRecord;
		this.attPrefix = attprefix;
	}

	/**
	 * Gibt DAV-Attribut-Prefix ('Gm' oder 'Ba') zur&uuml;ck.
	 * 
	 * @return DAV-Attribut-Prefix
	 */
	public String getDavAttributPrefix() {
		return attPrefix;
	}

	/**
	 * Gibt die Information zur&uuml;ck, ob es sich hier um einen Bauabschnitt
	 * handelt.
	 * 
	 * @return <code>true</code> wenn Bauabschnitt
	 */
	public boolean isBauabschnitt() {
		final AttributeType attributeType = getData().getAttributeType();
		return attributeType.getPid().equals("atl.mdmBisBauabschnitt");
	}

	/**
	 * Gibt die Arbeitsstellen-ID zur&uuml;ck.
	 * 
	 * @return Arbeitsstellen-ID
	 */
	public String getArbeitsstellenID() {
		return getData().getItem(getDavAttributPrefix() + "ArbeitsstellenID").asTextValue().getText();
	}

	@Override
	public SituationRecord getSituationRecord() throws Datex2BaustelleException {
		situationRecord.setId(buildID());
		situationRecord.setVersion(buildVersion());

		situationRecord.setSituationRecordCreationTime(buildSituationRecordCreationTime());
		situationRecord.setSituationRecordVersionTime(buildSituationRecordVersionTime());

		// Situation - SituationRecord – probabilityOfOccurance
		// Wahrscheinlichkeit der Maßnahme [1] = “certain”
		// (verpflichtende Angabe; andere Werte sind im Rahmen des
		// Baustellenprofils nicht vorgesehen)
		situationRecord.setProbabilityOfOccurrence(ProbabilityOfOccurrenceEnum.CERTAIN);

		// Severity
		final Data item = data.getItem(getDavAttributPrefix() + "VerkehrsBehinderung");
		situationRecord.setSeverity(Datex2SeverityEnum.getInstance(item.asUnscaledValue().intValue()));

		// validity
		// status wird aktuell nicht benutzt
		final Validity validity = getObjectFactory().createValidity();
		final OverallPeriod period = getObjectFactory().createOverallPeriod();
		period.setOverallStartTime(
				WerkzeugSammlung.konvertiereDatum(data.getTimeValue(getDavAttributPrefix() + "Beginn").getMillis()));
		period.setOverallEndTime(
				WerkzeugSammlung.konvertiereDatum(data.getTimeValue(getDavAttributPrefix() + "Ende").getMillis()));
		validity.setValidityTimeSpecification(period);

		// "suspended" wird nicht unterstützt (fachlicher Sinn unklar)
		validity.setValidityStatus(ValidityStatusEnum.DEFINED_BY_VALIDITY_TIME_SPEC);
		situationRecord.setValidity(validity);

		buildComments();

		// impact (nur bei Bauabschnitten)
		if (isBauabschnitt()) {
			buildImpact();
		}

		// UrlLinks
		final List<UrlLink> urlLinks = situationRecord.getUrlLink();
		final Array zusatzInfo = data.getArray(getDavAttributPrefix() + "ZusatzInfo");
		for (int i = 0; i < zusatzInfo.getLength(); i++) {
			final UrlLink link = datex2ObjectFactory.createUrlLink();
			link.setUrlLinkAddress(zusatzInfo.getItem(i).getItem("ZusatzInfoReferenz").asTextValue().getText());
			link.setUrlLinkDescription(Datex2MultilingualString.konvertiereString(
					zusatzInfo.getItem(i).getItem("ZusatzInfoBeschreibung").asTextValue().getText()));
			urlLinks.add(link);
		}

		ortsReferenzen = new Datex2Ortsreferenzen(data.getItem(getDavAttributPrefix() + "OrtsReferenzen"));
		situationRecord.setGroupOfLocations(ortsReferenzen.getOrtsReferenzen());

		return situationRecord;
	}

	private void buildComments() {
		// GeneralPublicComment
		String text = getData().getItem(getDavAttributPrefix() + "Kommentar").asTextValue().getText();
		if (text != null && !text.isEmpty()) {
			final Comment comment = datex2ObjectFactory.createComment();
			comment.setComment(Datex2MultilingualString.konvertiereString(text));
			// bei 'normalen' Kommentaren wird keine Typisierung unterstützt
			situationRecord.getGeneralPublicComment().add(comment);
		}

		if (isBauabschnitt()) {
			text = getData().getItem("BaUmleitungsStrecken").asTextValue().getText();
			if (text != null && !text.isEmpty()) {
				final Comment comment = datex2ObjectFactory.createComment();
				comment.setComment(Datex2MultilingualString.konvertiereString(text));
				// Umleitungen mit Typ 'ROUTE_RECOMMENDATION'
				final CommentExtensionType commentExtensionType = datex2ObjectFactory.createCommentExtensionType();
				final CommentExtended commentExtended = datex2ObjectFactory.createCommentExtended();
				commentExtended.setCommentType2(CommentType2Enum.ROUTE_RECOMMENDATION);
				commentExtensionType.setCommentExtended(commentExtended);
				comment.setCommentExtension(commentExtensionType);
				situationRecord.getGeneralPublicComment().add(comment);
			}
		}
	}

	private void buildImpact() throws Datex2BaustelleException {

		final Data item = getData().getItem("BaFahrstreifenGesperrt");

		final Impact impact = getObjectFactory().createImpact();

		boolean hasImpact = false;

		NumberValue unscaledValue = item.getItem("RestkapazitätProzent").asUnscaledValue();
		if (!unscaledValue.isState()) {
			impact.setCapacityRemaining(unscaledValue.floatValue());
			hasImpact = true;
		}

		unscaledValue = item.getItem("AnzahlFahrstreifenGesperrt").asUnscaledValue();
		if (!unscaledValue.isState()) {
			impact.setNumberOfLanesRestricted(BigInteger.valueOf(unscaledValue.intValue()));
			hasImpact = true;
		}

		unscaledValue = item.getItem("AnzahlFahrstreifenOhneBaustelle").asUnscaledValue();
		if (!unscaledValue.isState()) {
			impact.setOriginalNumberOfLanes(BigInteger.valueOf(unscaledValue.intValue()));
			hasImpact = true;
		}

		NumberValue scaledValue = item.getItem("MaxDurchfahrtsBreite").asScaledValue();
		if (!scaledValue.isState()) {
			impact.setResidualRoadWidth(scaledValue.floatValue());
			hasImpact = true;
		}

		// impactExtended
		boolean hasImpactExtension = false;
		final ImpactExtensionType impactExtensionType = datex2ObjectFactory.createImpactExtensionType();
		final ImpactExtended impactExtended = datex2ObjectFactory.createImpactExtended();
		impactExtensionType.setImpactExtended(impactExtended);

		scaledValue = item.getItem("MaxDurchfahrtsHöhe").asScaledValue();
		if (!scaledValue.isState()) {
			impactExtended.setResidualHeight(scaledValue.floatValue());
			hasImpactExtension = true;
		}

		String text = item.getItem("FahrbahnQuerschnitt").asTextValue().getText();
		if (text != null && !text.isEmpty()) {
			impactExtended.setLaneStatusCoded(text);
			hasImpactExtension = true;
		}

		final Array fsarray = item.getArray("Fahrstreifen");
		if (fsarray.getLength() > 0) {
			for (int i = 0; i < fsarray.getLength(); i++) {
				final int fswert = fsarray.getItem(i).asUnscaledValue().intValue();
				impactExtended.getClosedLanes().add(Datex2FahrstreifenEnum.getInstance(fswert).getDatexWert());
			}

			hasImpactExtension = true;
		}

		text = item.getItem("FahrstreifenGesperrtAllgemein").asTextValue().getText();
		if ("Ja".equals(text)) {
			impactExtended.setLanesRestricted(true);
			hasImpactExtension = true;
		} else {
			impactExtended.setLanesRestricted(false);
		}

		final int intValue = item.getItem("SeitenstreifenNutzung").asUnscaledValue().intValue();
		final Datex2SeitenstreifenEnum instance = Datex2SeitenstreifenEnum.getInstance(intValue);

		if (instance.getDatexWert() != null) {
			impactExtended.setHardShoulderStatus(instance.getDatexWert());
			hasImpactExtension = true;
		}

		if (buildLaneRestriction(impactExtended)) {
			hasImpactExtension = true;
		}

		if (hasImpact || hasImpactExtension) {
			if (hasImpactExtension) {
				impact.setImpactExtension(impactExtensionType);
			}

			situationRecord.setImpact(impact);
		}
	}

	private boolean buildLaneRestriction(final ImpactExtended impactExtended) throws Datex2BaustelleException {
		final Array feArray = getData().getItem("BaFahrstreifenEinschränkung").asArray();
		if (feArray.getLength() > 0) {
			// LaneRestriction
			if (feArray.getLength() > 0) {
				for (int i = 0; i < feArray.getLength(); i++) {
					final LaneRestriction laneRestriction = datex2ObjectFactory.createLaneRestriction();
					final int fswert = feArray.getItem(i).getItem("Fahrstreifen").asUnscaledValue().intValue();
					laneRestriction.setLane(Datex2FahrstreifenEnum.getInstance(fswert).getDatexWert());

					NumberValue scaledValue = feArray.getItem(i).getItem("MaxFahrzeugGewicht").asScaledValue();
					if (!scaledValue.isState()) {
						laneRestriction.setMaxVehicleWeight(scaledValue.floatValue());
					}

					scaledValue = feArray.getItem(i).getItem("MinDurchfahrtsBreite").asScaledValue();
					if (!scaledValue.isState()) {
						laneRestriction.setMinLaneWidth(scaledValue.floatValue());
					}

					impactExtended.getLaneRestriction().add(laneRestriction);
				}
			}
			return true;
		}

		return false;
	}

	private String buildID() {
		final String arbeitsstellenID = getArbeitsstellenID();

		if (arbeitsstellenID == null || arbeitsstellenID.isEmpty()) {
			return UUID.randomUUID().toString();
		}

		return arbeitsstellenID;
	}

	private String buildVersion() {
		return "" + resultData.getDataTime();
	}

	/**
	 * Bildet die SituationRecordCreationTime.
	 * <p>
	 * Wird aus dem validSince-Attribut des Baustellen-Objektes gebildet.
	 * </p>
	 * 
	 * @return {@link XMLGregorianCalendar}
	 */
	private XMLGregorianCalendar buildSituationRecordCreationTime() {
		final SystemObject object = resultData.getObject();
		return WerkzeugSammlung.konvertiereDatum(new Date(((DynamicObject) object).getValidSince()));
	}

	/**
	 * Bildet die SituationRecordVersionTime.
	 * <p>
	 * Wird aus dem Zeitstempel des Datensatzes gebildet.
	 * </p>
	 * 
	 * @return {@link XMLGregorianCalendar}
	 */
	private XMLGregorianCalendar buildSituationRecordVersionTime() {
		return WerkzeugSammlung.konvertiereDatum(new Date(resultData.getDataTime()));
	}

	/**
	 * Gibt die zugeh&ouml;rigen {@link Data} zur&uuml;ck.
	 * 
	 * @return {@link Data}
	 */
	public Data getData() {
		return data;
	}

	/**
	 * Gibt die 'ArtDerMassnahme' zur&uuml;ck.
	 * 
	 * @return {@link Datex2ArtDerMassnahmeEnum}
	 * @throws Datex2BaustelleException
	 *             wenn nicht zugeordnet werden kann
	 */
	public Datex2ArtDerMassnahmeEnum getDatex2ArtDerMassnahmeEnum() throws Datex2BaustelleException {
		final Data item = getData().getItem(getDavAttributPrefix() + "ArtDerMassnahme");
		final Datex2ArtDerMassnahmeEnum instance = Datex2ArtDerMassnahmeEnum
				.getInstance(item.asUnscaledValue().intValue());
		return instance;
	}

	/**
	 * Gibt die {@link ObjectFactory} zur&uuml;ck.
	 * 
	 * @return {@link ObjectFactory}
	 */
	public ObjectFactory getObjectFactory() {
		return datex2ObjectFactory;
	}
}
