/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl;

import java.math.BigInteger;
import java.util.Collection;
import java.util.Date;
import java.util.Map;

import javax.naming.ConfigurationException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.datatype.XMLGregorianCalendar;

import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.plugins.definition.PluginExport;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrswerte.ConfidentialityValueEnum;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrswerte.CountryEnum;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrswerte.D2LogicalModel;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrswerte.ElaboratedData;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrswerte.ElaboratedDataPublication;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrswerte.Exchange;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrswerte.HeaderInformation;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrswerte.InformationStatusEnum;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrswerte.InternationalIdentifier;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrswerte.LocationByReference;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrswerte.PayloadPublication;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrswerte.PercentageValue;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrswerte.PredefinedLocationVersionedReference;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrswerte.PredefinedNonOrderedLocationGroupVersionedReference;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrswerte.ReferenceSettings;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrswerte.SpeedValue;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrswerte.TrafficFlow;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrswerte.TrafficSpeed;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrswerte.VehicleCharacteristics;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrswerte.VehicleFlowValue;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrswerte.VehicleTypeEnum;

/**
 * Konvertierung von dynamischen Verkehrsdaten (QKfz, VKfz etc.) in das DATEX-II Format.
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */

public class MQRohdaten extends StandardKonvertierungDynamisch  {
	
  /** Pid des Typs, zu denen diese Konvertierung funktioniert. */
  private static final String TYP_MESSQUERSCHNITT = "typ.messQuerschnitt";
  
  /** Pid der Attributgruppe, mit der diese Konvertierung funktioniert. */
  private static final String ATG_VERKEHRSDATEN = "atg.verkehrsDatenKurzZeitMq";
  
  /**
   * Konstruktor.
   * 
   * @param plugin Referenz auf Plugin-Definitions-Klasse.
   */
  
  public MQRohdaten(final PluginExport plugin) {
    this.plugin = plugin;
  }
  
  /***************************************************************************
   * IN DEM FOLGENDEN ABSCHNITT WERDEN STRUKTURELL GLEICHE METHODEN GESAMMELT.
   * 
   * Diese Methoden werden in den einzelnen Plugins wiederholt - allerdings
   * beziehen sich die JAXB-Elemente trotz gleichen Namens dann auf
   * andere Klassen; eine nominale Gleichheit existiert also nicht.
   ***************************************************************************
   */

  /**
   * Gebe den in dieser Klasse verwendeten JAXBContext zurück. 
   * 
   * @throws JAXBException JAXBException
   * @see javax.xml.bind.JAXBContext#newInstance(String)
   * @return JAXBContext Verwendeter Kontext
   */

  @Override
  public JAXBContext getJAXBContext() throws JAXBException {
    if (jaxbContext == null) {
      jaxbContext = JAXBContext.newInstance(D2LogicalModel.class.getPackage().getName());
    }
    return jaxbContext;
  }

  /**
   * Setzt die allgemeinen Angaben zur Publikation (Exchange, Language, PublicationCreator).
   * 
   * @param d2l D2LogicalModel
   * @param payload Verwendeter Payload
   */

  private void setzeNationalIdentifier(final D2LogicalModel d2l, final PayloadPublication payload) {
    InternationalIdentifier internationalIdentifier = new InternationalIdentifier();
    internationalIdentifier.setCountry(CountryEnum.DE);
    internationalIdentifier.setNationalIdentifier(nationalIdentifier);
    
    // Setze Exchange
    
    Exchange exchange = new Exchange();
    exchange.setSupplierIdentification(internationalIdentifier);
    d2l.setExchange(exchange);

    // Setze Language
    payload.setLang("DE");

    payload.setPublicationCreator(internationalIdentifier);
  }

  /**
   * Setzt Koordinaten aus Data in ein PointCoordinates Objekt.
   * 
   * @param data
   *          Data
   * @return PointCoordinates
   */

 

  /***********************************************************************
   * ENDE DES ABSCHNITTS STRUKTURELL GLEICHER METHODEN
   ***********************************************************************
   */

  /** {@inheritDoc} */
  @Override
  public Object getDatexII(final Map<ObjekteAtgAsp, Collection<ResultData>> data) {
    String predefinedLocationVersion = holePredefinedLocationVersion();

    D2LogicalModel d2l = new D2LogicalModel();
    ElaboratedDataPublication payload = new ElaboratedDataPublication();
    d2l.setPayloadPublication(payload);

    setzeNationalIdentifier(d2l, payload);

    setzeHeaderInformation(payload);    
    
    setzeReferenceSettings(predefinedLocationVersion, payload);
    
    setzeZeitangaben(payload);
    
    for (ObjekteAtgAsp objekteAtgAsp : data.keySet()) {
      if (!objekteAtgAsp.getTyp().getPid().equals(TYP_MESSQUERSCHNITT) 
          || (!objekteAtgAsp.getAttributGruppe().getPid().equals(ATG_VERKEHRSDATEN))) {
        throw new IllegalArgumentException(logPrefix 
            + "Interner Fehler: Unpassende Konfiguration des Plugins: "
            + "Typ/Attributgruppe passt nicht zur dynamischen Konvertierung"); 
      }
      for (ResultData resultData : data.get(objekteAtgAsp)) {
        if (resultData.hasData()) { 
          try {
            LocationByReference locationByReference = new LocationByReference();
            PredefinedLocationVersionedReference predefinedLocationVersionedReference = 
                new PredefinedLocationVersionedReference();
            predefinedLocationVersionedReference.setId(resultData.getObject().getPid());
            predefinedLocationVersionedReference.setVersion(predefinedLocationVersion);
            locationByReference.setPredefinedLocationReference(predefinedLocationVersionedReference);
            VehicleCharacteristics vehicleCharacteristicsPkw = new VehicleCharacteristics();
            vehicleCharacteristicsPkw.getVehicleTypes().add(VehicleTypeEnum.CAR);
            VehicleCharacteristics vehicleCharacteristicsLkw = new VehicleCharacteristics();
            vehicleCharacteristicsLkw.getVehicleTypes().add(VehicleTypeEnum.LORRY);
            VehicleCharacteristics vehicleCharacteristicsKfz = new VehicleCharacteristics();
            vehicleCharacteristicsKfz.getVehicleTypes().add(VehicleTypeEnum.ANY_VEHICLE);

            // QPkw      
            long qPkw = WerkzeugSammlung.getLongValue(resultData, "QPkw", "Wert", SMALL_MINUS_LONG);
            ElaboratedData elaboratedDataQPkw = new ElaboratedData();
            TrafficFlow trafficFlowPkw = new TrafficFlow();
            trafficFlowPkw.setForVehiclesWithCharacteristicsOf(vehicleCharacteristicsPkw);
            trafficFlowPkw.setPertinentLocation(locationByReference);
            VehicleFlowValue vehicleFlowValuePkw = new VehicleFlowValue();
            if (qPkw >= 0) {
              vehicleFlowValuePkw.setVehicleFlowRate(BigInteger.valueOf(qPkw));
              trafficFlowPkw.setVehicleFlow(vehicleFlowValuePkw);
              elaboratedDataQPkw.setBasicData(trafficFlowPkw);
              payload.getElaboratedDatas().add(elaboratedDataQPkw);
            } else {
              LOGGER.fine(String.format("Wert für QPkw wird nicht gesetzt, da kleiner 0: %d", qPkw));
            } 
            
            //QLkw
            long qLkw = WerkzeugSammlung.getLongValue(resultData, "QLkw", "Wert", SMALL_MINUS_LONG);
            ElaboratedData elaboratedDataQLkw = new ElaboratedData();
            TrafficFlow trafficFlowLkw = new TrafficFlow(); 
            trafficFlowLkw.setForVehiclesWithCharacteristicsOf(vehicleCharacteristicsLkw);
            trafficFlowLkw.setPertinentLocation(locationByReference);
            VehicleFlowValue vehicleFlowValueLkw = new VehicleFlowValue();
            if (qLkw >= 0) {
              vehicleFlowValueLkw.setVehicleFlowRate(BigInteger.valueOf(qLkw));
              trafficFlowLkw.setVehicleFlow(vehicleFlowValueLkw);
              elaboratedDataQLkw.setBasicData(trafficFlowLkw);
              payload.getElaboratedDatas().add(elaboratedDataQLkw);
            } else {
              LOGGER.fine(String.format("Wert für QLkw wird nicht gesetzt, da kleiner 0: %d", qLkw));
            } 

            // QKfz && ALkw   
            long qKfz = WerkzeugSammlung.getLongValue(resultData, "QKfz", "Wert", SMALL_MINUS_LONG);
            long aLkw = WerkzeugSammlung.getLongValue(resultData, "ALkw", "Wert", SMALL_MINUS_LONG);
            ElaboratedData elaboratedDataQKfz = new ElaboratedData();
            TrafficFlow trafficFlowKfz = new TrafficFlow();
            trafficFlowKfz.setForVehiclesWithCharacteristicsOf(vehicleCharacteristicsKfz);
            trafficFlowKfz.setPertinentLocation(locationByReference);
            VehicleFlowValue vehicleFlowValueKfz = new VehicleFlowValue();
            PercentageValue percentageValue = new PercentageValue();
            if (qKfz >= 0) {
              vehicleFlowValueKfz.setVehicleFlowRate(BigInteger.valueOf(qKfz));
              trafficFlowKfz.setPercentageLongVehicles(percentageValue);
              trafficFlowKfz.setVehicleFlow(vehicleFlowValueKfz);
              elaboratedDataQKfz.setBasicData(trafficFlowKfz); 
              payload.getElaboratedDatas().add(elaboratedDataQKfz);
            } else {
              LOGGER.fine(String.format("Wert für QKfz wird nicht gesetzt, da kleiner 0: %d", qKfz));
            } 
            if (aLkw >= 0) {
              percentageValue.setPercentage(aLkw);
            } else {
              trafficFlowKfz.setPercentageLongVehicles(null);
            }

            //VPkw      
            long vPkw = WerkzeugSammlung.getLongValue(resultData, "VPkw", "Wert", SMALL_MINUS_LONG);
            ElaboratedData elaboratedDataVPkw = new ElaboratedData();
            TrafficSpeed trafficSpeedPkw = new TrafficSpeed();
            trafficSpeedPkw.setForVehiclesWithCharacteristicsOf(vehicleCharacteristicsPkw);
            trafficSpeedPkw.setPertinentLocation(locationByReference);
            SpeedValue speedValuePkw = new SpeedValue();
            if (vPkw >= 0) {
              speedValuePkw.setSpeed(vPkw);
              trafficSpeedPkw.setAverageVehicleSpeed(speedValuePkw);
              elaboratedDataVPkw.setBasicData(trafficSpeedPkw);
              payload.getElaboratedDatas().add(elaboratedDataVPkw);
            } else {
              LOGGER.fine(String.format("Wert für VPkw wird nicht gesetzt, da kleiner 0: %d", vPkw));
            } 

            //VLkw      
            long vLkw = WerkzeugSammlung.getLongValue(resultData, "VLkw", "Wert", SMALL_MINUS_LONG);
            ElaboratedData elaboratedDataVLkw = new ElaboratedData();
            TrafficSpeed trafficSpeedLkw = new  TrafficSpeed();
            trafficSpeedLkw.setForVehiclesWithCharacteristicsOf(vehicleCharacteristicsLkw);
            trafficSpeedLkw.setPertinentLocation(locationByReference);
            SpeedValue speedValueLkw = new SpeedValue();
            if (vLkw >= 0) {
              speedValueLkw.setSpeed(vLkw);
              trafficSpeedLkw.setAverageVehicleSpeed(speedValueLkw);
              elaboratedDataVLkw.setBasicData(trafficSpeedLkw);
              payload.getElaboratedDatas().add(elaboratedDataVLkw);
            } else {
              LOGGER.fine(String.format("Wert für VLkw wird nicht gesetzt, da kleiner 0: %d", vLkw));
            } 
            
            //Vkfz      
            long vKfz = WerkzeugSammlung.getLongValue(resultData, "VKfz", "Wert", SMALL_MINUS_LONG);
            ElaboratedData elaboratedDataVKfz = new ElaboratedData();
            TrafficSpeed trafficSpeedKfz = new TrafficSpeed();
            trafficSpeedKfz.setForVehiclesWithCharacteristicsOf(vehicleCharacteristicsKfz);
            trafficSpeedKfz.setPertinentLocation(locationByReference);
            SpeedValue speedValueKfz = new SpeedValue();
            if (vKfz >= 0) {
              speedValueKfz.setSpeed(vKfz);
              trafficSpeedKfz.setAverageVehicleSpeed(speedValueKfz);
              elaboratedDataVKfz.setBasicData(trafficSpeedKfz);
              payload.getElaboratedDatas().add(elaboratedDataVKfz);  
            } else {
              LOGGER.fine(String.format("Wert für VKfz wird nicht gesetzt, da kleiner 0: %d", vKfz));
            } 
          } catch (Exception e) {
            LOGGER.fine(logPrefix + "Daten nicht berücksichtigt.", e);
          }
        }
      }    
    }    


    if (payload.getElaboratedDatas().isEmpty()) {
      LOGGER.warning(logPrefix + "Kein valider Datensatz verfügbar, Publikation wird abgebrochen");
      return null;
    } else { 
      return d2l;
    }
  }
  
  /**
   * Setze die HeaderInformation des übergebenen Payloads.
   * 
   * @param payload Zu ergänzender Payload
   */

  private void setzeHeaderInformation(final ElaboratedDataPublication payload) {
    HeaderInformation headerInformation = new HeaderInformation();
    headerInformation.setConfidentiality(ConfidentialityValueEnum.NO_RESTRICTION);
    headerInformation.setInformationStatus(InformationStatusEnum.REAL);
    payload.setHeaderInformation(headerInformation);
  }

  /**
   * Setze allgemeine Referenzangaben (LocationGroupId, -Version etc.).
   * 
   * @param predefinedLocationVersion die zu verwendende Version
   * @param payload der zu modifizierende Payload
   */

  private void setzeReferenceSettings(final String predefinedLocationVersion, final ElaboratedDataPublication payload) {
    ReferenceSettings referenceSettings = new ReferenceSettings();
    PredefinedNonOrderedLocationGroupVersionedReference predefinedNonOrderedLocationGroupVersionedReference =
        new PredefinedNonOrderedLocationGroupVersionedReference();
    predefinedNonOrderedLocationGroupVersionedReference.setId(locationGroupId);
    predefinedNonOrderedLocationGroupVersionedReference.setVersion(predefinedLocationVersion);
    referenceSettings.setPredefinedNonOrderedLocationGroupReference(predefinedNonOrderedLocationGroupVersionedReference);
    payload.setReferenceSettings(referenceSettings);
  }

  /**
   * Setze allgemeine Zeitangaben (PublicationTime etc.).
   * 
   * @param payload der zu modifizierende Payload
   */

  private void setzeZeitangaben(final ElaboratedDataPublication payload) {
    Date datum = new Date();
    XMLGregorianCalendar xmlGregorianCalendar = WerkzeugSammlung.konvertiereDatum(datum);
 
    payload.setPublicationTime(xmlGregorianCalendar);

    payload.setPeriodDefault(intervall);
    payload.setTimeDefault(xmlGregorianCalendar);
  }


  /** {@inheritDoc} */
  @Override
  public void initialisiere(final Collection<ObjekteAtgAsp> objects) throws ConfigurationException {	
    initialisiereBasisFelder();
    initialisiereStandardIntervall();
  }

}
