package de.bsvrz.kex.datex2.plugins.konvertierung.impl;

import java.util.Collection;
import java.util.Map;

import javax.naming.ConfigurationException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;

import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.plugins.definition.PluginExport;
import de.bsvrz.kex.datex2.plugins.konvertierung.DatexKonvertierungDynamisch;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Abstrakte Klasse für gemeinsam genutzte Strukturen verschiedener Plugins.
 * 
 * @author TraffGo Road
 *
 */

public abstract class StandardKonvertierungDynamisch implements DatexKonvertierungDynamisch {
  /** Schlüssel Standard-Intervall. */
  protected static final String KEY_STANDARD_INTERVALL = "StandardIntervall";
  /** Schlüssel LocationGroupId. */
  protected static final String KEY_LOCATION_GROUP_ID = "LocationGroupId";
  /** Schlüssel NationalIdentifier. */
  protected static final String KEY_NATIONAL_IDENTIFIER = "National-Identifier";
  /** Referenz auf Plugin-Definitions-Klasse. */
  protected PluginExport plugin;
  /** Logging. */
  protected static final Debug LOGGER = Debug.getLogger();
  /** NationalIdentifier. */
  protected String nationalIdentifier;
  /** Präfix für Log-Ausgaben. */
  protected String logPrefix = "";
  /** Kontext für JAXB. */
  protected JAXBContext jaxbContext;
  /** Versionsstring, falls Verknüpfung zu statischer Konvertierung nicht hergestellt werden kann. */
  private static final String FAIL_VERSION_STRING = "unknown";
  
  /** Standard-Messintervall (in Sekunden). */
  static final int STANDARD_INTERVALL = 60;
  
  /** Messintervall. */
  protected float intervall;
  

  /** Referenz auf statische Konvertierungsmethode. */
  protected StandardKonvertierungStatisch statischeKonvertierung;
  
  /** Id der LocationGroup. */
  protected String locationGroupId;
  
  /** Konstante für Fehlerwerte. */
  protected static final long SMALL_MINUS_LONG = -1;
  /** Konstante für Fehlerwerte. */
  protected static final float SMALL_MINUS_FLOAT = -1f;


    
  /**
   * Zugriff auf die PredefinedLocation Tabellenversion über die statische Konvertierungsmethode.
   *
   * @return Tabellenversion oder FAIL_VERSION_STRING, falls kein Zugriff möglich ist.
   */
  
  public String holePredefinedLocationVersion() {
    String version = FAIL_VERSION_STRING;
    if (statischeKonvertierung != null) {
      String letzteVerwendeteVersion = statischeKonvertierung.getPredefinedLocationVersion();
      if ((letzteVerwendeteVersion != null) && (!letzteVerwendeteVersion.isEmpty())) {
        version = letzteVerwendeteVersion;
      }
    }

    return version;
  }
  
  /**
   * Initialisierung von grundlegenden Feldern (NationalIdentifier, Log-Präfix).
   * 
   * @throws ConfigurationException Wird geworfen, wenn Pflichtfelder nicht eingelesen werden konnten.
   */
  protected void initialisiereBasisFelder() throws ConfigurationException {
    logPrefix = WerkzeugSammlung.erzeugeLogPrefix(plugin);

    nationalIdentifier = WerkzeugSammlung.holeKonfigurationsdatum(plugin, KEY_NATIONAL_IDENTIFIER);

    locationGroupId = WerkzeugSammlung.holeKonfigurationsdatum(plugin, KEY_LOCATION_GROUP_ID);
    
    try {
      statischeKonvertierung = (StandardKonvertierungStatisch) plugin.getDatexKonvertierungStatisch();
    } catch (Exception e) {
      LOGGER.warning(logPrefix + "Zugriff auf statische Konvertierungsdaten nicht möglich, " 
          + "setzte Version der PredefinedLocation auf " + FAIL_VERSION_STRING + ".");
    }
  }
  
  /**
   * Initialisierung des Standard-Intervalls (Datex-II spezifische Größe).
   */

  protected void initialisiereStandardIntervall() {
    intervall = STANDARD_INTERVALL;
    if (plugin.getKonfigurationsModul().hasData(KEY_STANDARD_INTERVALL)) {
      String standardIntervall = plugin.getKonfigurationsModul().getString(KEY_STANDARD_INTERVALL);
      try {
        intervall = Float.parseFloat(standardIntervall);
      } catch (Exception e) {
        LOGGER.warning(logPrefix + "StandardIntervall konnte nicht eingelesen werden (" 
            + standardIntervall + "), nutze Standard-Wert", e);
      }
    } else {
      LOGGER.error(logPrefix + "Das Feld '" + KEY_STANDARD_INTERVALL + "' wurde nicht in der Konfiguration angegeben, " 
                + "nutze Standard-Wert (" + intervall + " Sekunden)");      
    }
  }
  
  @Override
  public abstract Object getDatexII(Map<ObjekteAtgAsp, Collection<ResultData>> data);
  
  @Override
  public abstract JAXBContext getJAXBContext() throws JAXBException;
  
  @Override
  public abstract void initialisiere(Collection<ObjekteAtgAsp> objects) throws ConfigurationException;


}