/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Map;

import javax.naming.ConfigurationException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.plugins.definition.PluginExport;
import de.bsvrz.kex.datex2.plugins.konvertierung.DatexKonvertierungDynamisch;
import de.bsvrz.sys.funclib.debug.Debug;
import eu.datex2.schema.dynamisch.tmcmeldungen.AbnormalTraffic;
import eu.datex2.schema.dynamisch.tmcmeldungen.AbnormalTrafficTypeEnum;
import eu.datex2.schema.dynamisch.tmcmeldungen.AlertCDirection;
import eu.datex2.schema.dynamisch.tmcmeldungen.AlertCDirectionEnum;
import eu.datex2.schema.dynamisch.tmcmeldungen.AlertCLocation;
import eu.datex2.schema.dynamisch.tmcmeldungen.AlertCMethod2Linear;
import eu.datex2.schema.dynamisch.tmcmeldungen.AlertCMethod2Point;
import eu.datex2.schema.dynamisch.tmcmeldungen.AlertCMethod2PrimaryPointLocation;
import eu.datex2.schema.dynamisch.tmcmeldungen.AlertCMethod2SecondaryPointLocation;
import eu.datex2.schema.dynamisch.tmcmeldungen.ConfidentialityValueEnum;
import eu.datex2.schema.dynamisch.tmcmeldungen.CountryEnum;
import eu.datex2.schema.dynamisch.tmcmeldungen.D2LogicalModel;
import eu.datex2.schema.dynamisch.tmcmeldungen.Exchange;
import eu.datex2.schema.dynamisch.tmcmeldungen.HeaderInformation;
import eu.datex2.schema.dynamisch.tmcmeldungen.InformationStatusEnum;
import eu.datex2.schema.dynamisch.tmcmeldungen.InternationalIdentifier;
import eu.datex2.schema.dynamisch.tmcmeldungen.Linear;
import eu.datex2.schema.dynamisch.tmcmeldungen.ObjectFactory;
import eu.datex2.schema.dynamisch.tmcmeldungen.OverallPeriod;
import eu.datex2.schema.dynamisch.tmcmeldungen.Point;
import eu.datex2.schema.dynamisch.tmcmeldungen.ProbabilityOfOccurrenceEnum;
import eu.datex2.schema.dynamisch.tmcmeldungen.Situation;
import eu.datex2.schema.dynamisch.tmcmeldungen.SituationExtended;
import eu.datex2.schema.dynamisch.tmcmeldungen.SituationExtensionType;
import eu.datex2.schema.dynamisch.tmcmeldungen.SituationPublication;
import eu.datex2.schema.dynamisch.tmcmeldungen.TMCEvent;
import eu.datex2.schema.dynamisch.tmcmeldungen.Validity;
import eu.datex2.schema.dynamisch.tmcmeldungen.ValidityStatusEnum;

/**
 * Konvertierungsklasse für TMC-Meldungen aus dem Datenverteiler in das DATEX-II Format.
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */

public class TMCMeldungen implements DatexKonvertierungDynamisch {
  
  /**
   * Konstanten EventCodes.
   * 
   * @author TraffGo Road GmbH
   *
   */
  public class EvCodes {
    /** Stau. */
    private static final int STAU = 101;
    /** Stau. */
    private static final int STAU_1KM = 102;
    /** Stau. */
    private static final int STAU_2KM = 103;
    /** Stau. */
    private static final int STAU_3KM = 129;
    /** Stau. */
    private static final int STAU_5KM = 104;
    /** Stau. */
    private static final int STAU_7KM = 105;
    /** Stau. */
    private static final int STAU_10KM = 106;
    /** Stockender Verkehr. */
    private static final int STOCKEND = 108;
    /** Stockender Verkehr. */
    private static final int STOCKEND_1KM = 109;
    /** Stockender Verkehr. */
    private static final int STOCKEND_2KM = 110;
    /** Stockender Verkehr. */
    private static final int STOCKEND_3KM = 131;
    /** Stockender Verkehr. */
    private static final int STOCKEND_5KM = 111;
    /** Stockender Verkehr. */
    private static final int STOCKEND_7KM = 112;
    /** Stockender Verkehr. */
    private static final int STOCKEND_10KM = 113;
  }
  
  /** Schlüssel für AlertC Table Nummer. */
  private static final String KEY_ALERTC_TABLE_NR = "AlertC-Table-Number";
  
  /** Schlüssel für AlertC Table Version. */
  private static final String KEY_ALERTC_TABLE_VERSION = "AlertC-Table-Version";
  
  /** Schlüssel für AlertC Table Country Code. */
  private static final String KEY_ALERTC_TABLE_COUNTRY = "AlertC-Table-Country";

  /** Schlüssel für NationalIdentifier. */
  private static final String KEY_NATIONAL_IDENTIFIER = "National-Identifier";
  
  /** Akzeptierte Attributgruppe. */
  private static final Object ATG_RDSMELDUNG = "atg.rdsMeldung";
  
  /** National-Identifier. */
  private String nationalIdentifier;
  
  /** Referenz auf Plugin-Definitions-Klasse. */
  private PluginExport plugin;
  
  /** Logging. */
  private static final Debug LOGGER = Debug.getLogger();
  
  /** Event-Codes für Stau. */
  private List<Integer> stationaryTrafficCodes = new ArrayList<>();
  // = Arrays.asList(EvCodes.STAU, 102, 103, 129, 104, 105, 106);

  /** Event-Codes für stockenden Verkehr. */
  private List<Integer> queueingTrafficCodes = new ArrayList<>(); 
  // = Arrays.asList(108, 109, 110, 131, 111, 112, 113);

  /** AlertC Table Nummer. */
  private String alertCTableNumber = "111130";
  
  /** AlertC Table Version. */
  private String alertCTableVersion = "11.0";
  
  /** AlertC Table Country Code. */
  private String alertCTableCountry = "DE";

  /** Präfix für Log-Ausgaben. */
  private String logPrefix;
 
  /** Kontext für JAXB. */
  protected JAXBContext jaxbContext;

  /** 
   * Konstruktor.
   * 
   * @param plugin Referenz auf Plugin-Definitions-Klasse
   */
  public TMCMeldungen(final PluginExport plugin) {
    this.plugin = plugin;
    
    stationaryTrafficCodes.add(EvCodes.STAU);
    stationaryTrafficCodes.add(EvCodes.STAU_1KM);
    stationaryTrafficCodes.add(EvCodes.STAU_2KM);
    stationaryTrafficCodes.add(EvCodes.STAU_3KM);
    stationaryTrafficCodes.add(EvCodes.STAU_5KM);
    stationaryTrafficCodes.add(EvCodes.STAU_7KM);
    stationaryTrafficCodes.add(EvCodes.STAU_10KM);
    
    queueingTrafficCodes.add(EvCodes.STOCKEND);
    queueingTrafficCodes.add(EvCodes.STOCKEND_1KM);
    queueingTrafficCodes.add(EvCodes.STOCKEND_2KM);
    queueingTrafficCodes.add(EvCodes.STOCKEND_3KM);
    queueingTrafficCodes.add(EvCodes.STOCKEND_5KM);
    queueingTrafficCodes.add(EvCodes.STOCKEND_7KM);
    queueingTrafficCodes.add(EvCodes.STOCKEND_10KM);
  }

  /***************************************************************************
   * IN DEM FOLGENDEN ABSCHNITT WERDEN STRUKTURELL GLEICHE METHODEN GESAMMELT.
   * 
   * Diese Methoden werden in den einzelnen Plugins wiederholt - allerdings
   * beziehen sich die JAXB-Elemente trotz gleichen Namens dann auf
   * andere Klassen; eine nominale Gleichheit existiert also nicht.
   ***************************************************************************
   */

  /**
   * Gebe den in dieser Klasse verwendeten JAXBContext zurück. 
   * 
   * @throws JAXBException JAXBException
   * @see javax.xml.bind.JAXBContext#newInstance(String)
   * @return JAXBContext Verwendeter Kontext
   */

  @Override
  public JAXBContext getJAXBContext() throws JAXBException {
    if (jaxbContext == null) {
      jaxbContext = JAXBContext.newInstance(D2LogicalModel.class.getPackage().getName());
    }
    return jaxbContext;
  }

  /**
   * Setzt die allgemeinen Angaben zur Publikation (Exchange, Language, PublicationCreator).
   * 
   * @param d2l D2LogicalModel
   */

  private void setzeNationalIdentifier(final D2LogicalModel d2l) {
    InternationalIdentifier internationalIdentifier = new InternationalIdentifier();
    internationalIdentifier.setCountry(CountryEnum.DE);
    internationalIdentifier.setNationalIdentifier(nationalIdentifier);

    // Setze Exchange

    Exchange exchange = new Exchange();
    exchange.setSupplierIdentification(internationalIdentifier);
    d2l.setExchange(exchange);

    // Setze Language
    d2l.getPayloadPublication().setLang("DE");

    d2l.getPayloadPublication().setPublicationCreator(internationalIdentifier);
  }

  /***********************************************************************
   * ENDE DES ABSCHNITTS STRUKTURELL GLEICHER METHODEN
   ***********************************************************************
   */

  /** {@inheritDoc} */
  @Override
  public Object getDatexII(final Map<ObjekteAtgAsp, Collection<ResultData>> data) {
    D2LogicalModel model = null;
    GregorianCalendar cal = (GregorianCalendar) GregorianCalendar.getInstance();

    // Initialize Factories

    ObjectFactory objectFactory = new ObjectFactory();

    model = objectFactory.createD2LogicalModel();

    // Schema-Angaben

    model.setExtensionName(model.getExtensionName());
    model.setExtensionVersion(model.getExtensionVersion());

    // SituationPublication

    SituationPublication situationPublication = objectFactory
        .createSituationPublication();
    model.setPayloadPublication(situationPublication);

    // SituationPublication/publicationTime

    cal.setTime(new Date());
    situationPublication.setPublicationTime(WerkzeugSammlung.konvertiereDatum(new Date()));

    setzeNationalIdentifier(model);
    
    for (ObjekteAtgAsp objekteAtgAsp : data.keySet()) {
      if (!objekteAtgAsp.getAttributGruppe().getPid().equals(ATG_RDSMELDUNG)) {
        LOGGER.warning(logPrefix 
            + "Unpassende Konfiguration des Plugins: Typ/Attributgruppe passt nicht zur dynamischen Konvertierung"); 
      } else {
        for (ResultData resultData : data.get(objekteAtgAsp)) {
          if ((resultData != null) && (resultData.hasData())) {
            try { 
              boolean canBeTransformed = true;
              String infoGUID = resultData.getData().getItem("InfoGUID").asTextValue().getText();
              String versionGUID = resultData.getData().getItem("Version").getItem("VersionGUID").asTextValue().getText();
              Array events = (Array) resultData.getData()
                  .getItem("Version")
                  .getItem("VerkehrsInformationen")
                  .getItem("Ereignis")
                  .getItem("EreignisDaten").asArray();
              List<Integer> usefulEvents = new ArrayList<>();

              AbnormalTrafficTypeEnum type = null;

              for (int i = 0; i < events.getLength(); i++) {
                int evcode = events.getItem(i).getItem("EreignisCode").asUnscaledValue().intValue();
                if (stationaryTrafficCodes.contains(evcode)) {
                  usefulEvents.add(evcode);
                  type = AbnormalTrafficTypeEnum.STATIONARY_TRAFFIC;
                }
                if (queueingTrafficCodes.contains(evcode)) {
                  usefulEvents.add(evcode);
                  if (type != AbnormalTrafficTypeEnum.STATIONARY_TRAFFIC) {
                    type = AbnormalTrafficTypeEnum.QUEUING_TRAFFIC;
                  }
                }
              }
              canBeTransformed = !usefulEvents.isEmpty();

              GregorianCalendar firstPublish = (GregorianCalendar) GregorianCalendar.getInstance();
              GregorianCalendar latestPublish = (GregorianCalendar) GregorianCalendar.getInstance();
              GregorianCalendar startTime = (GregorianCalendar) GregorianCalendar.getInstance();
              GregorianCalendar endTime = (GregorianCalendar) GregorianCalendar.getInstance();
              Data versionInfo = resultData.getData().getItem("Version").getItem("VerwaltungsInformationen");
              firstPublish.setTimeInMillis(versionInfo.getItem("AktivierungsZeit").asTimeValue().getMillis());
              latestPublish.setTimeInMillis(versionInfo.getItem("AktualisierungsZeit").asTimeValue().getMillis());
              startTime.setTimeInMillis(versionInfo.getItem("ErzeugungsZeit").asTimeValue().getMillis());
              endTime.setTimeInMillis(versionInfo.getItem("AblaufZeit").asTimeValue().getMillis());

              String alertCDirection = resultData.getData().
                  getItem("Version").
                  getItem("VerkehrsInformationen").
                  getItem("LocationDaten").
                  getTextValue("TMCRichtung").getText();

              int locationMethod = resultData.getData()
                  .getItem("Version")
                  .getItem("VerkehrsInformationen")
                  .getItem("LocationDaten")
                  .getItem("LocationMethode")
                  .asUnscaledValue().intValue();
              Array locations = resultData.getData()
                  .getItem("Version")
                  .getItem("VerkehrsInformationen")
                  .getItem("LocationDaten")
                  .getArray("RDSLocation");

              int locationPrimary = 0;
              int locationSecondary = 0;

              if (locations.getLength() > 0) {
                locationPrimary = locations.getItem(0).getUnscaledValue("LocationCode").intValue();
                locationSecondary = locations.getItem(locations.getLength() - 1).getUnscaledValue("LocationCode").intValue();
              } else {
                LOGGER.warning("Skipping, because LocationMethod unsupported.");            
                canBeTransformed = false;
              }

              if (locationMethod != 0) {
                canBeTransformed = false;
                LOGGER.warning(logPrefix + "Skipping, because LocationMethod unsupported.");
              }
              if (canBeTransformed) {

                // SituationPublication/Situation/SituationRecord

                Situation situation = objectFactory.createSituation();

                AbnormalTraffic abnormalTraffic = objectFactory.createAbnormalTraffic();
                situation.getSituationRecords().add(abnormalTraffic);
                abnormalTraffic.setAbnormalTrafficType(type);

                // Set Id+Version (nach xsd)

                situation.setId(infoGUID);
                situation.setVersion(versionGUID);

                abnormalTraffic.setId(infoGUID);
                abnormalTraffic.setVersion(versionGUID);

                // HeaderInformation

                HeaderInformation headerInformation = objectFactory
                    .createHeaderInformation();
                situation.setHeaderInformation(headerInformation);

                // InformationStatus

                headerInformation.setInformationStatus(InformationStatusEnum.REAL);

                // Confidentiality

                headerInformation
                .setConfidentiality(ConfidentialityValueEnum.NO_RESTRICTION);

                // Probability of occurence

                abnormalTraffic
                .setProbabilityOfOccurrence(ProbabilityOfOccurrenceEnum.CERTAIN);

                // SituationExtended/TMCEvent

                SituationExtensionType extType = objectFactory
                    .createSituationExtensionType();
                situation.setSituationExtension(extType);

                SituationExtended ext = objectFactory.createSituationExtended();
                extType.setSituationExtended(ext);

                TMCEvent tmc = objectFactory.createTMCEvent();
                ext.setTmcEvent(tmc);

                for (Integer ev : usefulEvents) {
                  tmc.getEventCodes().add(BigInteger.valueOf(ev));
                }
                ext.setTmcEvent(tmc);

                // Zeitstempel der ersten Version

                abnormalTraffic.setSituationRecordCreationTime(WerkzeugSammlung.konvertiereDatum(firstPublish));

                abnormalTraffic.setSituationRecordVersionTime(WerkzeugSammlung.konvertiereDatum(latestPublish));

                // GeneralComment: Freitexte

                // Validity/OverallPeriod

                Validity validity = objectFactory.createValidity();
                abnormalTraffic.setValidity(validity);

                OverallPeriod overallPeriod = objectFactory.createOverallPeriod();
                validity.setValidityTimeSpecification(overallPeriod);

                validity
                .setValidityStatus(ValidityStatusEnum.DEFINED_BY_VALIDITY_TIME_SPEC);

                // Validity: StartTime

                overallPeriod.setOverallStartTime(WerkzeugSammlung.konvertiereDatum(startTime));

                // Validity: EndTime

                overallPeriod.setOverallEndTime(WerkzeugSammlung.konvertiereDatum(endTime));

                AlertCDirection acd = new AlertCDirection();
                switch (alertCDirection) {
                case "Positiv": acd.setAlertCDirectionCoded(AlertCDirectionEnum.POSITIVE); break;
                case "Negativ": acd.setAlertCDirectionCoded(AlertCDirectionEnum.NEGATIVE); break;
                default: acd.setAlertCDirectionCoded(AlertCDirectionEnum.UNKNOWN);
                }

                AlertCMethod2PrimaryPointLocation primaryLocation = objectFactory
                    .createAlertCMethod2PrimaryPointLocation();

                AlertCLocation priLoc = objectFactory.createAlertCLocation();
                primaryLocation.setAlertCLocation(priLoc);

                priLoc.setSpecificLocation(BigInteger.valueOf(locationPrimary));

                AlertCMethod2SecondaryPointLocation secondaryLocation = objectFactory
                    .createAlertCMethod2SecondaryPointLocation();

                AlertCLocation secLoc = objectFactory.createAlertCLocation();
                secondaryLocation.setAlertCLocation(secLoc);

                secLoc.setSpecificLocation(BigInteger.valueOf(locationSecondary));

                if (locationPrimary == locationSecondary) {
                  Point point = objectFactory.createPoint();

                  AlertCMethod2Point acp = objectFactory.createAlertCMethod2Point();
                  point.setAlertCPoint(acp);

                  // AlertC: LCL

                  acp.setAlertCLocationCountryCode(alertCTableCountry);
                  acp.setAlertCLocationTableNumber(alertCTableNumber);
                  acp.setAlertCLocationTableVersion(alertCTableVersion);
                  acp.setAlertCDirection(acd);
                  acp.setAlertCMethod2PrimaryPointLocation(primaryLocation);
                  
                  abnormalTraffic.setGroupOfLocations(point);
                } else {
                  Linear linear = objectFactory.createLinear();

                  // AlertC Linear

                  AlertCMethod2Linear acl = objectFactory.createAlertCMethod2Linear();
                  linear.setAlertCLinear(acl);

                  // AlertC: LCL

                  acl.setAlertCLocationCountryCode(alertCTableCountry);
                  acl.setAlertCLocationTableNumber(alertCTableNumber);
                  acl.setAlertCLocationTableVersion(alertCTableVersion);
                  acl.setAlertCDirection(acd);

                  // AlertC: Secondary Location

                  acl.setAlertCMethod2SecondaryPointLocation(secondaryLocation);
                  acl.setAlertCMethod2PrimaryPointLocation(primaryLocation);

                  abnormalTraffic.setGroupOfLocations(linear);
                }
                situationPublication.getSituations().add(situation);
              }
            } catch (Exception e) {
              LOGGER.warning(logPrefix + "Meldung nicht berücksichtigt.", e);
            }
          }
        }
      }
    }
    return model;
  }

  @Override
  public void initialisiere(final Collection<ObjekteAtgAsp> objects) throws ConfigurationException {
    logPrefix = WerkzeugSammlung.erzeugeLogPrefix(plugin);

    alertCTableNumber = WerkzeugSammlung.holeKonfigurationsdatum(plugin, KEY_ALERTC_TABLE_NR);
    alertCTableVersion = WerkzeugSammlung.holeKonfigurationsdatum(plugin, KEY_ALERTC_TABLE_VERSION);
    alertCTableCountry = WerkzeugSammlung.holeKonfigurationsdatum(plugin, KEY_ALERTC_TABLE_COUNTRY); 

    nationalIdentifier = WerkzeugSammlung.holeKonfigurationsdatum(plugin, KEY_NATIONAL_IDENTIFIER);
  }
}
