/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl;


import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.GregorianCalendar;

import javax.naming.ConfigurationException;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.kex.datex2.plugins.definition.PluginExport;
import de.bsvrz.kex.datex2.plugins.konvertierung.DatexKonvertierungStatisch.CreationOptions;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Allgemeine Klasse für einfache Hilfsmethoden.
 */

public final class WerkzeugSammlung {
  
  /**
   * Statische Klasse - nicht instantiierbar.
   */
  private WerkzeugSammlung() {
  }
	
  /** Logging. */
  private static final Debug LOGGER = Debug.getLogger();

  /** Factory für XMLGregorianCalendar. */
	private static DatatypeFactory datatypeFactory;
	   
	/**
	 * Initialisiere die Klasse.
	 * 
	 * @return true, wenn Klasse ordnungsgemäß initialisiert ist/wurde.
	 */
	
	static {
	  try {
	    datatypeFactory = DatatypeFactory.newInstance();
	  } catch (DatatypeConfigurationException e) {
	    LOGGER.error("Fehler bei der Initialisierung der Datatype-Klasse. Mögliche Ursachen:" 
	        + "a) Falsche System-Property javax.xml.datatype.DatatypeFactory" 
	        + "b) Falsche Einträge in Datei ${JAVA_HOME}/lib/jaxp.properties" 
	        + "c) META-INF/services/java.xml.datatype.DatatypeFactory", e);
	    throw new IllegalArgumentException("Fehler bei der Initialisierung der Datatype-Klasse.");
	  }
	}
	  
	/**
	   * Konvertiere Date nach XMLCalendar.
	   * 
	   * @param zeitStempel
	   *          Zeitstempel
	   * @return Zeitstempel als XMLCalendar Objekt
	   */
	
	 public static XMLGregorianCalendar konvertiereDatum(final Date zeitStempel) {
		 GregorianCalendar gregorianCalendar = new GregorianCalendar();
		 gregorianCalendar.setTime(zeitStempel);
     return datatypeFactory.newXMLGregorianCalendar(gregorianCalendar);
	 }
	
	
	/**
	   * Konvertiere Date nach XMLCalendar.
	   * 
	   * @param millis
	   *          Millisekunden
	   * @return Zeitstempel als XMLCalendar Objekt
	   */
	
	public static XMLGregorianCalendar konvertiereDatum(final long millis) {
		    GregorianCalendar gregorianCalendar = new GregorianCalendar();
		    gregorianCalendar.setTimeInMillis(millis);
		    XMLGregorianCalendar xmlGregorianCalendar = null;
		    xmlGregorianCalendar = datatypeFactory.newXMLGregorianCalendar(gregorianCalendar);
		    return xmlGregorianCalendar;
		  }
	
	/**
	   * extrahiert einen String aus der zweiten Ebene von ResultData.
	   * 
	   * @param data
	   *          ResultData
	   * @param field
	   *          Name des Feldes
	   * @param key
	   *          Name des Wertes
	   * @return String
	   *
	   */
	
	public static String getStringValue(final ResultData data, final String field, final String key) {
		String svalue = "";
	    try {
	    	svalue = data.getData().getItem(field).getTextValue(key).getValueText();
	    } catch (Exception e) {
	    	LOGGER.fine("Fehler beim lesen von String " + key + " unter " + field);
	    }
	    return svalue;
	  }
	
	/**
	   * extrahiert einen String aus der zweiten Ebene von ResultData.
	   * 
	   * @param data
	   *          ResultData
	   * @param key
	   *          Name des Wertes
	   * @return String
	   *
	   */
	
	public static String getStringValue(final Data data, final String key) {
		String svalue = "";
	    try {
	    	svalue = data.getTextValue(key).getValueText();
	    } catch (Exception e) {
	    	LOGGER.fine("Fehler beim Lesen von String " + key);
	    }
	    return svalue;
	  }
	
	
	
	/**
	   * extrahiert einen Zahlwert (long) aus der zweiten Ebene von ResultData.
	   * 
	   * @param data
	   *          ResultData
	   * @param field
	   *          Name des Feldes
	   * @param key
	   *          Name des Wertes
	   * @param defaultValue
	   *          Default Wert, falls ein Fehler bei der Zahlenkonvertierung auftritt
	   * @return Zahl als long
	   */
	
	public static long getLongValue(final ResultData data, final String field, final String key, final long defaultValue) {
	    long res = defaultValue;
      Data item = data.getData().getItem(field);
	    try {
	    	res = item.getScaledValue(key).longValue();
	    } catch (Exception e) {
	    	LOGGER.fine("Fehler beim Lesen von Long " + key + " unter " + field + " (Rohdatum: " + item);
	    }
	   
	    return res;
	  }
	
	
	
	/**
	   * extrahiert einen Zahlwert (long) aus der zweiten Ebene von ResultData. 
	   * 
	   * @param data
	   *          ResultData
	   * @param key
	   *          Name des Wertes
	   * @param defaultValue
	   *          Default Wert, falls ein Fehler bei der Zahlenkonvertierung auftritt
	   * @return Zahl als long
	   */
	
	public static long getLongValue(final ResultData data, final String key, final long defaultValue) {
	    long res = defaultValue;
	    try {
	    	String svalue = data.getData().getScaledValue(key).getValueText();
	    	 try {
	 	    	res = Long.parseLong(svalue);
	 	    } catch (NumberFormatException e) {
	 	    	res = defaultValue;
	 	    }
	    } catch (Exception e) {
	    	LOGGER.fine("Fehler beim Lesen von Long " + key);
	    }
	   
	    return res;
	  }
	
	
	/**
	   * extrahiert einen Zahlwert (long) aus der zweiten Ebene von ResultData. 
	   * 
	   * @param data
	   *          ResultData
	   * @param key
	   *          Name des Wertes
	   * @param defaultValue
	   *          Default Wert, falls ein Fehler bei der Zahlenkonvertierung auftritt
	   * @return Zahl als long
	   */
	
	public static long getLongValue(final Data data, final String key, final long defaultValue) {
	    long res = defaultValue;
	    try {
	    	String svalue = data.getUnscaledValue("key").getText();
	    	
	    	 try {
	 	    	res = Long.parseLong(svalue);
	 	    } catch (NumberFormatException e) {
	 	    	res = defaultValue;
	 	    }
	    } catch (Exception e) {
	    	LOGGER.fine("Fehler beim Lesen von Long " + key);
	    }
	   
	    return res;
	  }
	
	/**
	   * extrahiert einen Fließkommawert (float) aus der zweiten Ebene von ResultData. 
	   * 
	   * @param data
	   *          ResultData
	   * @param field
	   *          Name des Feldes
	   * @param key
	   *          Name des Wertes
	   * @param defaultValue
	   *          Default Wert, falls ein Fehler bei der Zahlenkonvertierung auftritt
	   * @param roundTo
	   *          Nachkommastelle, auf die gerundet werden soll.
	   * @return Zahl als float
	   */
	
	public static float getFloatValue(final ResultData data, final String field, final String key, 
	    final float defaultValue, final int roundTo) {
	    float res = defaultValue;
	    try {
	      double tmp = data.getData().getItem(field).getScaledValue(key).doubleValue();
	      res = new BigDecimal(tmp).setScale(roundTo, RoundingMode.HALF_UP).floatValue();
	    } catch (Exception e) {
	    	LOGGER.fine("Fehler beim lesen von Float " + key + " unter " + field + "; nutze Default-Wert.");
	    }
	    return res;
	  }
	
	
	/**
	   * extrahiert einen Fließkommawert (float) aus der ersten Ebene von ResultData.
	   * 
	   * @param data
	   *          ResultData
	   * @param key
	   *          Name des Wertes
	   * @param defaultValue
	   *          Default Wert, falls ein Fehler bei der Zahlenkonvertierung auftritt
	   * @return Zahl als float
	   */
	
	public static float getFloatValue(final ResultData data, final String key, final float defaultValue) {
	    float res = defaultValue;
	    try {
	    String svalue = data.getData().getScaledValue(key).getValueText();
		    try {
		    	res = Float.parseFloat(svalue.replace(",", "."));
		    } catch (NumberFormatException e) {
		      res = defaultValue;
		    }
	    } catch (Exception e) {
	    	LOGGER.fine("Fehler beim lesen von Float " + key);
	    }
	    return res;
	  }
	
	/**
	 * Konvertiere long nach int.
	 * 
	 * @param longValue long-Wert
	 * @return Integer-Wert
	 * @throws IllegalArgumentException Falls Zahl außerhalb Integer-Wertebereich.
	 */
  public static int getIntValue(final long longValue) throws IllegalArgumentException {
    if (longValue < Integer.MIN_VALUE || longValue > Integer.MAX_VALUE) {
      throw new IllegalArgumentException("Ganzzahlwert zu groß");
    }
    return (int) longValue;
  }
  
  /**
   * Konvertiere Calendar-Objekt in XML-Calendar-Objekt.
   * 
   * @param arg0 GregorianCalendar Instanz
   * @return XMLGregorianCalendar-Instanz
   */

  public static XMLGregorianCalendar konvertiereDatum(final GregorianCalendar arg0) {
    return datatypeFactory.newXMLGregorianCalendar(arg0);
  }
  
  /**
   * Erzeuge einen String für die Verwendung als Versionsangabe in statischen Publikationen.
   * 
   * @param versionFree true, wenn keine Versionsinformationen verwendet werden sollen).
   * @return Versionsstring aus aktuellem Datum oder leerer String, wenn versionFree = true
   */

  public static String erzeugeVersionsString(final boolean versionFree) {
    if (versionFree) {
      return "";
    } else {
      DateFormat formatter = new SimpleDateFormat("yyyyMMddHHmm");
      return formatter.format(new Date());
    }
  }

  /**
   * Erzeuge ein Datum für die Verwendung als Versionsangabe in statischen Publikationen.
   * 
   * @param versionFree true, wenn keine Versionsinformationen verwendet werden sollen).
   * @return Versionsstring aus aktuellem Datum oder die Epoche, wenn versionFree = true
   */

  public static XMLGregorianCalendar erzeugeVersionsdatum(final boolean versionFree) {
    if (versionFree) {
      return konvertiereDatum(1);
    } else {
      return konvertiereDatum(new Date());
    } 
  }

  /**
   * Liefert true zurück, wenn in Liste der Optionen "VERSION_FREE" auftritt.
   * 
   * @param options Liste der Optionen
   * @return true, wenn VERSION_FREE auftritt.
   */
  public static boolean isVersionFree(final CreationOptions... options) {
    boolean res = false;
    for (int i = 0; i < options.length; i++) {
      if (options[i].equals(CreationOptions.VERSION_FREE)) {
        res = true;
      }
    }
    return res;
   }
  
  /**
   * Runden einer Zahl auf einer bestimmten Nachkommastelle (für float).
   * 
   * @param zahl Die zu rundende Zahl.
   * @param stellen Nachkommastelle, bei der gerundet wird.
   * @return Gerundete Zahl.
   */

  public static float rundeZahl(final float zahl, final int stellen) {
    return new BigDecimal(zahl).setScale(stellen, RoundingMode.HALF_UP).floatValue();
  }
  

  /**
   * Runden einer Zahl auf einer bestimmten Nachkommastelle (für double).
   * 
   * @param zahl Die zu rundende Zahl.
   * @param stellen Nachkommastelle, bei der gerundet wird.
   * @return Gerundete Zahl.
   */

  public static double rundeZahl(final double zahl, final int stellen) {
    return new BigDecimal(zahl).setScale(stellen, RoundingMode.HALF_UP).floatValue();
  }
  
  /**
   * Holt die Geokoordinte aus einer Datenverteiler-Struktur mit atg.punktKoordinaten.
   * 
   * @param data Referenz auf DaV-Datenstruktur der atg.punktKoordinaten
   * @return Latitude ("y"-Wert).
   */
  
  public static float getLatitude(final Data data) {
    String posLat = data.getScaledValue("y").getValueText();
    float fLat = 0f;
    try {
      fLat = Float.parseFloat(posLat.replace(",", "."));
    } catch (NumberFormatException nfe) {
      LOGGER.warning("Die Koordinate '" + posLat + "' konnte nicht verwendet werden.");
    }
    return fLat;
  }

  /**
   * Holt die Geokoordinte aus einer Datenverteiler-Struktur mit atg.punktKoordinaten.
   * 
   * @param data Referenz auf DaV-Datenstruktur der atg.punktKoordinaten
   * @return Longitude ("x"-Wert).
   */
  
  public static float getLongitude(final Data data) {
    String posLong = data.getScaledValue("x").getValueText();
    float fLong = 0f;
    try {
      fLong = Float.parseFloat(posLong.replace(",", "."));
    } catch (NumberFormatException nfe) {
      LOGGER.warning("Die Koordinate '" + posLong + "' konnte nicht verwendet werden.");
    }
    return fLong;
  }
  
  /**
   * Erzeuge einen zum Plugin passenden Präfix für Logging-Einträge.
   * 
   * @param plugin Referenz auf Plugin.
   * @return String mit Log-Präfix.
   */

  public static String erzeugeLogPrefix(final PluginExport plugin) {
    return "In Kontext '" + plugin.getKonfigurationsModul().getContext() + "': ";
  }
  
  /**
   * Liest ein Konfigurationsdatum aus der im Plugin referenzierten Konfiguration mit dem Schlüssel "key"
   * als String.
   * 
   * @param plugin Referenz auf das Plugin
   * @param key Schlüssel des Konfigurationsdatums
   * @return Wert
   * @throws ConfigurationException wenn der Schlüssel nicht konfiguriert ist oder ein anderer Fehler aufgetreten ist.
   */

  public static String holeKonfigurationsdatum(final PluginExport plugin, final String key) throws ConfigurationException {
    if (plugin.getKonfigurationsModul().hasData(key)) {
      return plugin.getKonfigurationsModul().getString(key);
    } else {
      throw new ConfigurationException(erzeugeLogPrefix(plugin) + "Das Feld '" + key 
          + "' wurde nicht in der Konfiguration angegeben.");
    }  
  }

  public static boolean pruefeMinutenIntervall(int aktuelleMinute, int minute) {
    return aktuelleMinute % minute == 0;
  }

}
