/*
 * Plug-in KEx DATEX II - Baustellen
 * Copyright (C) 2018 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl.elemente;

import de.bsvrz.kex.datex2.plugins.konvertierung.impl.Datex2BaustelleException;
import eu.datex2.schema.dynamisch.baustellen.OperatorActionStatusEnum;

/**
 * Umsetzung Datk <code>att.mdmBisStatus</code> in Datex2
 * <code>OperatorActionStatusEnum</code>.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: $
 */

public enum Datex2ActionStatusEnum {

	/**
	 * keine Angabe&nbsp;0&nbsp;Es liegen keine Angaben zum Status vor (DATEX
	 * II: keine Entsprechung).
	 */
	KeineAngabe(null, 0),

	/**
	 * Planung&nbsp;1&nbsp;Maßnahme in Planung (DATEX II: requested).
	 */
	Planung(OperatorActionStatusEnum.REQUESTED, 1),

	/**
	 * Angeordnet&nbsp;2&nbsp;Maßnahme verkehrsrechtlich angeordnet (DATEX II:
	 * approved).
	 */
	Angeordnet(OperatorActionStatusEnum.APPROVED, 2),

	/**
	 * Durchführung&nbsp;3&nbsp;Maßnahme wird durchgeführt (DATEX II:
	 * beeingImplemented).
	 */
	Durchfuehrung(OperatorActionStatusEnum.BEING_IMPLEMENTED, 3),

	/**
	 * Durchgeführt&nbsp;4&nbsp;Maßnahme wurde durchgeführt (ggf. werden aber
	 * noch Verkehrsbehinderungen verursacht) (DATEX II: implemented).
	 */
	Durchgefuehrt(OperatorActionStatusEnum.IMPLEMENTED, 4),

	/**
	 * Zurückgewiesen&nbsp;5&nbsp;Maßnahme wurde nicht angeordnet /
	 * zurückgewiesen (DATEX II: rejected).
	 */
	Zurueckgewiesen(OperatorActionStatusEnum.REJECTED, 5),

	/**
	 * Soll beendet werden&nbsp;6&nbsp;Maßnahme soll beendet werden (DATEX II:
	 * terminationRequested).
	 */
	SollBeendetWerden(OperatorActionStatusEnum.TERMINATION_REQUESTED, 6),

	/**
	 * Wird beendet&nbsp;7&nbsp;Maßnahme wird beendet (DATEX II:
	 * beeingTerminated).
	 */
	WirdBeendet(OperatorActionStatusEnum.BEING_TERMINATED, 7),

	/**
	 * Abgeschlossen&nbsp;8&nbsp;Maßnahme abgeschlossen (DATEX II: keine
	 * Entsprechung, da beendete Baustellen nicht übertragen werden).
	 */
	Abgeschlossen(null, 8);

	/**
	 * Gibt eine passende Instanz zur&uuml;ck.
	 *
	 * @param davwert
	 *            DAV-Wert
	 * @return zum numerischen Wert passende Instanz
	 * @throws Datex2BaustelleException
	 *             bei ung&uuml;ltigem Wert
	 */
	public static Datex2ActionStatusEnum getInstance(final int davwert) throws Datex2BaustelleException {
		for (Datex2ActionStatusEnum adm : Datex2ActionStatusEnum.values()) {
			if (adm.getDavId() == davwert) {
				return adm;
			}
		}
		throw new Datex2BaustelleException(
				"Kein Datex2-'Datex2ActionStatusEnum' zu DAV-Wert=" + davwert + " definiert.");
	}

	/**
	 * Datex2-Wert.
	 */
	private final OperatorActionStatusEnum datexWert;

	/**
	 * numerischer DatK- Wert.
	 */
	private int david;

	/**
	 * Privater Konstruktor.
	 *
	 * @param value
	 *            textueller Wert
	 * @param id
	 *            numerischer Wert
	 */
	Datex2ActionStatusEnum(final OperatorActionStatusEnum value, final int id) {
		this.datexWert = value;
		david = id;
	}

	/**
	 * Gibt den numerischen Wert zur&uuml;ck.
	 *
	 * @return numerischer Wert der Himmelsrichtung
	 */
	public int getDavId() {
		return david;
	}

	/**
	 * Gibt den DatexII-Wert zur&uuml;ck.
	 * 
	 * @return {@link OperatorActionStatusEnum}
	 */
	public OperatorActionStatusEnum getDatexWert() {
		return datexWert;
	}

}
