/*
 * Plug-in KEx DATEX II - Baustellen
 * Copyright (C) 2018 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl.elemente;

import de.bsvrz.kex.datex2.plugins.konvertierung.impl.Datex2BaustelleException;
import eu.datex2.schema.dynamisch.baustellen.LaneEnum;

/**
 * Umsetzung Datk <code>att.mdmBisFahrstreifen</code> in Datex2
 * <code>LaneEnum</code>.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: $
 */
public enum Datex2FahrstreifenEnum {

	/**
	 * Seitenstreifen&nbsp;0&nbsp;Seitenstreifen (DATEX II: hardshoulder).
	 */
	Seitenstreifen(LaneEnum.HARD_SHOULDER, 0),

	/**
	 * FS 1 &nbsp;1&nbsp;1er Fahrstreifen von rechts - ohne Seitenstreifen!
	 * (DATEX II: lane1).
	 */
	FS1(LaneEnum.LANE_1, 1),

	/**
	 * FS 2 &nbsp;2&nbsp;2er Fahrstreifen von rechts - ohne Seitenstreifen!
	 * (DATEX II: lane2).
	 */
	FS2(LaneEnum.LANE_2, 2),

	/**
	 * FS 3 &nbsp;3&nbsp;3er Fahrstreifen von rechts - ohne Seitenstreifen!
	 * (DATEX II: lane3).
	 */
	FS3(LaneEnum.LANE_3, 3),

	/**
	 * FS 4 &nbsp;4&nbsp;4er Fahrstreifen von rechts - ohne Seitenstreifen!
	 * (DATEX II: lane4).
	 */
	FS4(LaneEnum.LANE_4, 4),

	/**
	 * FS 5 &nbsp;5&nbsp;5er Fahrstreifen von rechts - ohne Seitenstreifen!
	 * (DATEX II: lane5).
	 */
	FS5(LaneEnum.LANE_5, 5),

	/**
	 * FS 6 &nbsp;6&nbsp;6er Fahrstreifen von rechts - ohne Seitenstreifen!
	 * (DATEX II: lane6).
	 */
	FS6(LaneEnum.LANE_6, 6),

	/**
	 * FS 7 &nbsp;7&nbsp;7er Fahrstreifen von rechts - ohne Seitenstreifen!
	 * (DATEX II: lane7).
	 */
	FS7(LaneEnum.LANE_7, 7),

	/**
	 * FS 8 &nbsp;8&nbsp;8er Fahrstreifen von rechts - ohne Seitenstreifen!
	 * (DATEX II: lane8).
	 */
	FS8(LaneEnum.LANE_8, 8),

	/**
	 * FS 9 &nbsp;9&nbsp;9er Fahrstreifen von rechts - ohne Seitenstreifen!
	 * (DATEX II: lane9).
	 */
	FS9(LaneEnum.LANE_9, 9),

	/**
	 * FS links &nbsp;10&nbsp;Linker Fahrstreifen - sollte nur verwendet werden,
	 * wenn die relative Position des FS nicht genauer bekannt ist (DATEX II:
	 * leftLange).
	 */
	FSLINKS(LaneEnum.LEFT_LANE, 10),

	/**
	 * FS rechts &nbsp;11&nbsp; Rechter Fahrstreifen - sollte nur verwendet
	 * werden, wenn die relative Position des FS nicht genauer bekannt ist
	 * (DATEX II: rightLange).
	 */
	FSRECHTS(LaneEnum.RIGHT_LANE, 11),

	/**
	 * FS gesamt &nbsp;12&nbsp; Gesamte Fahrbahn, jedoch ohne den
	 * Seitenstreifen! (DATEX II: allLanesCompleteCarriageway).
	 */
	FSGESAMT(LaneEnum.ALL_LANES_COMPLETE_CARRIAGEWAY, 12);

	/**
	 * Gibt eine passende Instanz zur&uuml;ck.
	 *
	 * @param d2wert
	 *            Datex2-Wert
	 * @return zum numerischen Wert passende Instanz
	 * @throws Datex2BaustelleException
	 *             bei ung&uuml;ltigem Wert
	 */
	public static Datex2FahrstreifenEnum getInstance(final LaneEnum d2wert) throws Datex2BaustelleException {
		for (Datex2FahrstreifenEnum adm : Datex2FahrstreifenEnum.values()) {
			if (adm.getDatexWert() != null && adm.getDatexWert().equals(d2wert)) {
				return adm;
			}
		}
		throw new Datex2BaustelleException("Keine Datex2-'LaneEnum' mit XML-Wert=" + d2wert.toString() + " definiert.");
	}

	/**
	 * Gibt eine passende Instanz zur&uuml;ck.
	 *
	 * @param davwert
	 *            DaV-Wert
	 * @return zum numerischen Wert passende Instanz
	 * @throws Datex2BaustelleException
	 *             bei ung&uuml;ltigem Wert
	 */
	public static Datex2FahrstreifenEnum getInstance(final int davwert) throws Datex2BaustelleException {
		for (Datex2FahrstreifenEnum adm : Datex2FahrstreifenEnum.values()) {
			if (adm.getDatexWert() != null && adm.getDavId() == davwert) {
				return adm;
			}
		}
		throw new Datex2BaustelleException("Keine Datex2-'LaneEnum' zu DAV-Wert=" + davwert + " definiert.");
	}

	/**
	 * Datex2-Wert.
	 */
	private final LaneEnum datexWert;

	/**
	 * Gibt den DatexII-Wert zur&uuml;ck.
	 * 
	 * @return {@link LaneEnum}
	 */
	public LaneEnum getDatexWert() {
		return datexWert;
	}

	/**
	 * numerischer Wert.
	 */
	private int david;

	/**
	 * Privater Konstruktor.
	 *
	 * @param value
	 *            textueller Wert
	 * @param id
	 *            numerischer Wert
	 */
	Datex2FahrstreifenEnum(final LaneEnum value, final int david) {
		this.datexWert = value;
		this.david = david;
	}

	/**
	 * Gibt den numerischen Wert zur&uuml;ck.
	 *
	 * @return numerischer Wert der Himmelsrichtung
	 */
	public int getDavId() {
		return david;
	}
}
