/*
 * Plug-in KEx DATEX II - Baustellen
 * Copyright (C) 2018 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl.elemente;

import de.bsvrz.kex.datex2.plugins.konvertierung.impl.Datex2BaustelleException;
import eu.datex2.schema.dynamisch.baustellen.HardShoulderStatusEnum;

/**
 * Umsetzung Datk <code>att.mdmBisSeitenstreifenNutzung</code> in Datex2
 * <code>HardShoulderStatusEnum</code>.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: $
 */
public enum Datex2SeitenstreifenEnum {

	/**
	 * keine Angabe&nbsp;0&nbsp;Zur Nutzung des Seitenstreifens liegen keine
	 * Angaben vor.(DATEX II: hardshoulder).
	 */
	KeineAngabe(null, 0),

	/**
	 * als Fahrstreifen nutzbar&nbsp;1&nbsp;Seitenstreifen kann als Fahrstreifen
	 * genutzt werden (DATEX II: availableForDriving).
	 */
	AlsFahrstreifenNutzbar(HardShoulderStatusEnum.AVAILABLE_FOR_DRIVING, 1),

	/**
	 * als Pannenstreifen nutzbar&nbsp;2&nbsp;Seitenstreifen kann nur als
	 * Pannenstreifen genutzt werden (DATEX II: availableForStopping).
	 */
	AlsPannenstreifenNutzbar(HardShoulderStatusEnum.AVAILABLE_FOR_STOPPING, 2),

	/**
	 * gesperrt&nbsp;3&nbsp;Seitenstreifen ist gesperrt und kann nicht genutzt
	 * werden (DATEX II: closed).
	 */
	Gesperrt(HardShoulderStatusEnum.CLOSED, 3);

	/**
	 * Gibt eine passende Instanz zur&uuml;ck.
	 *
	 * @param d2wert
	 *            Datex2-Wert
	 * @return zum numerischen Wert passende Instanz
	 * @throws Datex2BaustelleException
	 *             bei ung&uuml;ltigem Wert
	 */
	public static Datex2SeitenstreifenEnum getInstance(final HardShoulderStatusEnum d2wert)
			throws Datex2BaustelleException {
		for (Datex2SeitenstreifenEnum adm : Datex2SeitenstreifenEnum.values()) {
			if (adm.getDatexWert() != null && adm.getDatexWert().equals(d2wert)) {
				return adm;
			}
		}
		throw new Datex2BaustelleException(
				"Keine Datex2-'HardShoulderStatusEnum' mit XML-Wert=" + d2wert.toString() + " definiert.");
	}

	/**
	 * Gibt eine passende Instanz zur&uuml;ck.
	 *
	 * @param davwert
	 *            DaV-Wert
	 * @return zum numerischen Wert passende Instanz
	 * @throws Datex2BaustelleException
	 *             bei ung&uuml;ltigem Wert
	 */
	public static Datex2SeitenstreifenEnum getInstance(final int davwert) throws Datex2BaustelleException {
		for (Datex2SeitenstreifenEnum adm : Datex2SeitenstreifenEnum.values()) {
			if (adm.getDavId() == davwert) {
				return adm;
			}
		}
		throw new Datex2BaustelleException(
				"Keine Datex2-'HardShoulderStatusEnum' zu DAV-Wert=" + davwert + " definiert.");
	}

	/**
	 * Datex2-Wert.
	 */
	private final HardShoulderStatusEnum datexWert;

	/**
	 * Gibt den DatexII-Wert zur&uuml;ck.
	 * 
	 * @return {@link HardShoulderStatusEnum}
	 */
	public HardShoulderStatusEnum getDatexWert() {
		return datexWert;
	}

	/**
	 * numerischer Wert.
	 */
	private int david;

	/**
	 * Privater Konstruktor.
	 *
	 * @param value
	 *            textueller Wert
	 * @param id
	 *            numerischer Wert
	 */
	Datex2SeitenstreifenEnum(final HardShoulderStatusEnum value, final int david) {
		this.datexWert = value;
		this.david = david;
	}

	/**
	 * Gibt den numerischen Wert zur&uuml;ck.
	 *
	 * @return numerischer Wert der Himmelsrichtung
	 */
	public int getDavId() {
		return david;
	}
}
